@@ -168,7 +168,7 @@ asgtk_dir_tree_expand_handler(GtkTreeView *treeview,
 
 		asgtk_dir_tree_refresh_child( GTK_TREE_STORE(model), &child_iter, fullpath, 1 );
 		gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter, TREE_SCANNED, 1, -1);
-	}while(gtk_tree_model_iter_next (model, &child_iter));	 
+	}while(gtk_tree_model_iter_next (model, &child_iter));	
 }
 
 /*  public functions  */
@@ -365,6 +365,18 @@ void  asgtk_dir_tree_refresh( ASGtkDirTree *dt )
 			}
 			curr = curr->next ;
 		}
+                /*
+		   This section doesnt seem to be needed here... ?
+                        :sG:
+                        The following 3 lines make the file lists sorted by
+                        the File Name, Ascending.
+                        
+                        For general file listing (not dirs)
+                GtkTreeSortable *sortable;
+                sortable = GTK_TREE_SORTABLE(dt->tree_model);
+                gtk_tree_sortable_set_sort_column_id(sortable,1,GTK_SORT_ASCENDING);
+		*/
+
 	}		   
 	if( curr_sel ) 
 		free( curr_sel );
