@@ -94,14 +94,14 @@ asgtk_image_browser_class_init (ASGtkImageBrowserClass *klass)
 }
 
 static void
-asgtk_image_browser_init (ASGtkImageBrowser *id)
+asgtk_image_browser_init (ASGtkImageBrowser *ib)
 {
 }
 
 static void
 asgtk_image_browser_dispose (GObject *object)
 {
-  	/*ASGtkImageBrowser *ib = ASGTK_IMAGE_BROWSER (object); */
+  	/* ASGtkImageBrowser *ib = ASGTK_IMAGE_BROWSER (object); */
   	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -260,18 +260,15 @@ asgtk_image_browser_create_dir_list( ASGtkImageBrowser *ib, GtkWidget *list_vbox
 	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (ib->dir_list)), "changed",
 		    		  G_CALLBACK (asgtk_image_browser_dir_select), ib);
 	
-  	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
-  	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_win), GTK_SHADOW_IN);  
-  	gtk_container_add (GTK_CONTAINER (scrolled_win), ib->dir_list);
-  	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
-									GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-  	gtk_container_set_border_width (GTK_CONTAINER (scrolled_win), 0);
+  	scrolled_win = ASGTK_SCROLLED_WINDOW(GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS,GTK_SHADOW_IN);
+
+	ASGTK_CONTAINER_CONFIG(scrolled_win,DIR_LIST_WIDTH, DIR_LIST_HEIGHT,0);
+	ASGTK_CONTAINER_ADD(scrolled_win, ib->dir_list);
 	if (GTK_IS_PANED (list_vbox))
 		gtk_paned_pack1 (GTK_PANED (list_vbox), GTK_WIDGET(scrolled_win), TRUE, TRUE);
 	else
 		gtk_box_pack_end (GTK_BOX (list_vbox), GTK_WIDGET(scrolled_win), TRUE, TRUE, 0);
   		
-  	gtk_widget_show (ib->dir_list);
   	gtk_widget_show (scrolled_win);
 	colorize_gtk_tree_view_window( GTK_WIDGET(scrolled_win) );
 }
@@ -407,7 +404,6 @@ asgtk_image_browser_new ()
 	
 	gtk_button_set_alignment( GTK_BUTTON(ib->scale_check_box), 1.0, 0.5);
 	asgtk_image_view_add_detail( ib->preview, ib->scale_check_box, 0 );
-
 	/* changing to default dir : */
 	asgtk_image_dir_set_sel_handler( ASGTK_IMAGE_DIR(ib->image_dir), asgtk_image_browser_file_sel_handler, ib);
 	asgtk_image_browser_change_dir( ib, NULL );	
@@ -449,7 +445,6 @@ asgtk_image_browser_refresh( ASGtkImageBrowser *ib )
 	asgtk_image_dir_set_path(ASGTK_IMAGE_DIR(ib->image_dir), ib->current_dir);
 	dir_store = GTK_LIST_STORE (gtk_tree_view_get_model(GTK_TREE_VIEW(ib->dir_list)));	
 	gtk_list_store_clear(dir_store);
-   		
 	if( ib->current_dir != NULL ) 
 	{
 		/* Add current dir entry as the first item :*/
@@ -512,6 +507,17 @@ asgtk_image_browser_refresh( ASGtkImageBrowser *ib )
 		gtk_list_store_append (dir_store, &iter);
 		gtk_list_store_set (dir_store, &iter, 0, "/", -1);
 	}
+               /*
+                        :sG:
+                        The following 3 lines make the file lists sorted by
+                        the Directory Name, Ascending.
+
+			For general directory listing (not files)
+                */
+                GtkTreeSortable *sortable;
+                sortable = GTK_TREE_SORTABLE(dir_store);
+                gtk_tree_sortable_set_sort_column_id(sortable,0,GTK_SORT_ASCENDING);
+
 	
 }
 
