@@ -649,6 +649,110 @@ destroy_ASModule_config( ASModuleConfig *config )
 	}
 }
 
+MyStyleDefinition*
+free_storage_elem2MyStyleDefinition (FreeStorageElem *fs, const char *default_name)
+{
+	MyStyleDefinition *msd = NULL;
+	
+	if (fs == NULL)
+		return NULL;
+
+	msd = safecalloc (1, sizeof (MyStyleDefinition));
+	msd->type = CONFIG_MyStyles_ID;
+	
+	if (fs->sub)
+		fs = fs->sub;
+	for ( ;fs ; fs = fs->next)
+	{
+		TermDef *T = fs->term ;
+		
+		if (T->id < MYSTYLE_ID_START || T->id > MYSTYLE_ID_END) /* just in case */
+			continue;
+
+		if (T->struct_field_offset != 0) 
+		{
+			ReadConfigItemToStruct( msd, offsetof(MyStyleDefinition,set_flags), fs  );
+/* fprintf (stderr, "$$$$ name = %s, offset=%d,keyword =%s,flags_on=0x%X, set_flags = 0x%X\n",msd->Name, offsetof(MyStyleDefinition,set_flags), T->keyword, T->flags_on, msd->set_flags); */
+		}else if( T->type == TT_FLAG)
+		{
+			ReadFlagItemAuto (msd, offsetof(MyStyleDefinition,set_flags), fs, MyStyleFlags);	
+		}else
+			HandleMyStyleSpecialTerm (msd, fs);
+	}	
+	
+	if (msd->Name == NULL && default_name)
+		msd->Name = mystrdup (default_name);
+	
+	if (!CheckMyStyleDefinitionSanity (msd))
+		DestroyMyStyleDefinitions (&msd);
+		
+	return msd;
+}
+
+FreeStorageElem *
+MyStyleDefinition2free_storage_elem (MyStyleDefinition *msd, SyntaxDef *syntax)
+{
+	FreeStorageElem *fs = NULL;
+
+	if (msd == NULL || syntax == NULL)
+		return NULL;
+
+	if (msd->Name == NULL)
+		return NULL;
+
+	QuotedString2FreeStorage (syntax, &fs, NULL, msd->Name, MYSTYLE_MyStyle_ID);
+	if (fs)
+	{
+		FreeStorageElem **tail = &(fs->sub);
+		ASFlagType auto_handled = 0, auto_handled_flags = 0;
+		TermDef *T = fs->term;
+		
+		fs->sub = StructToFreeStorage (msd, offsetof (MyStyleDefinition, set_flags), T->sub_syntax, &auto_handled);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+
+		/* now we need to handle some flags : */
+		*tail = StructFlags2FreeStorage (msd, offsetof(MyStyleDefinition,set_flags), T->sub_syntax, MyStyleFlags, &auto_handled_flags);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+
+		/* now we need to handle some special items : */
+		*tail = MyStyleSpecialTerms2FreeStorage (msd, T->sub_syntax);
+	}		
+	return fs;
+}
+
+
+MyStyleDefinition*
+free_storage2MyStyleDefinitionsList (FreeStorageElem *fs)
+{
+	MyStyleDefinition *head = NULL;
+	MyStyleDefinition **tail = &head;
+	
+    for (; fs; fs = fs->next)
+	{
+		if (fs->term->id == MYSTYLE_MyStyle_ID)
+		{
+			*tail = free_storage_elem2MyStyleDefinition (fs->sub, NULL);
+			ADVANCE_LINKED_LIST_TAIL(tail);
+		}	
+	}
+	return head;	
+}
+
+FreeStorageElem *
+MyStyleDefinitionsList2free_storage (MyStyleDefinition *msd, SyntaxDef *syntax)
+{
+	FreeStorageElem *head = NULL;
+	FreeStorageElem **tail = &head;
+	
+    for (; msd; msd = msd->next)
+	{
+		*tail = MyStyleDefinition2free_storage_elem (msd, syntax);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+	}
+	return head;	
+}
+
+
 ASModuleConfig* 
 free_storage2ASModule_config( ASModuleConfigClass *class, ASModuleConfig *config, FreeStorageElem *Storage, ASFlagType flags )
 {
@@ -719,10 +823,12 @@ free_storage2ASModule_config( ASModuleConfigClass *class, ASModuleConfig *config
 							config = defaults ;
 						}
 					}				
-				}else if( T->id == MYSTYLE_START_ID && 
+				}else if( T->id == MYSTYLE_MyStyle_ID && 
 				          get_flags( flags, ASModuleConfig_HandleMyStyles ) )
 				{
-             		styles_tail = ProcessMyStyleOptions (pCurr->sub, styles_tail);
+             		*styles_tail = free_storage_elem2MyStyleDefinition (pCurr->sub, NULL);
+					if (*styles_tail)
+						styles_tail = &((*styles_tail)->next); 
 				}else if( T->struct_field_offset != 0  && 
 				         get_flags( flags, ASModuleConfig_HandleScalars )) 
 				{
@@ -852,3 +958,8 @@ parse_asmodule_config_all(ASModuleConfigClass *class )
 
 	return config;
 }
+
+/******************************************************************************
+ * Test cases and utilities
+ ******************************************************************************/
+
