@@ -574,111 +574,3 @@ LOCAL_DEBUG_OUT( "gravity = %d, default_gravity = %d", Config->gravity, default_
 }
 
 
-#if 0
-/* returns:
- *            0 on success
- *              1 if data is empty
- *              2 if toWriter cannot be initialized
- *
- */
-int
-WriteWinListOptions (const char *filename, char *myname, WinListto * config, unsigned long flags)
-{
-	toDef    *PagertoWriter = NULL;
-	FreeStorageElem *Storage = NULL, **tail = &Storage;
-	char         *Decorations = NULL;
-
-	if (config == NULL)
-		return 1;
-	if ((PagertoWriter = InittoWriter (myname, &PagerSyntax, CDT_Filename, (void *)filename)) == NULL)
-		return 2;
-
-	CopyFreeStorage (&Storage, config->more_stuff);
-
-    if (config->balloon_conf)
-		tail = balloon2FreeStorage (&WinListSyntax, tail, config->balloon_conf);
-
-	/* building free storage here */
-	/* geometry */
-	tail = Geometry2FreeStorage (&PagerSyntax, tail, &(config->geometry), PAGER_GEOMETRY_ID);
-	/* icon_geometry */
-	tail = Geometry2FreeStorage (&PagerSyntax, tail, &(config->icon_geometry), PAGER_ICON_GEOMETRY_ID);
-	/* labels */
-	tail = StringArray2FreeStorage (&PagerSyntax, tail, config->labels, desk1, desk2, PAGER_LABEL_ID);
-	/* styles */
-	tail = StringArray2FreeStorage (&PagerSyntax, tail, config->styles, desk1, desk2, PAGER_STYLE_ID);
-	/* align */
-	tail = Integer2FreeStorage (&PagerSyntax, tail, config->align, PAGER_ALIGN_ID);
-	/* flags */
-	if (!(config->flags & REDRAW_BG))
-		tail = Flag2FreeStorage (&PagerSyntax, tail, PAGER_DRAW_BG_ID);
-	if (config->flags & START_ICONIC)
-		tail = Flag2FreeStorage (&PagerSyntax, tail, PAGER_START_ICONIC_ID);
-	if (config->flags & FAST_STARTUP)
-		tail = Flag2FreeStorage (&PagerSyntax, tail, PAGER_FAST_STARTUP_ID);
-	if (config->flags & SET_ROOT_ON_STARTUP)
-		tail = Flag2FreeStorage (&PagerSyntax, tail, PAGER_SET_ROOT_ID);
-	if (config->flags & STICKY_ICONS)
-		tail = Flag2FreeStorage (&PagerSyntax, tail, PAGER_STICKY_ICONS_ID);
-
-	/* small_font_name */
-	tail = String2FreeStorage (&PagerSyntax, tail, config->small_font_name, PAGER_SMALL_FONT_ID);
-	/* rows */
-	tail = Integer2FreeStorage (&PagerSyntax, tail, config->rows, PAGER_ROWS_ID);
-	/* columns */
-	tail = Integer2FreeStorage (&PagerSyntax, tail, config->columns, PAGER_COLUMNS_ID);
-
-	/* now storing PagerDecorations */
-	{
-		toDef    *Decorto = InittoWriter (myname, &PagerDecorationSyntax, CDT_Data, NULL);
-		FreeStorageElem *DecorStorage = NULL, **d_tail = &DecorStorage;
-
-		/* flags */
-		if (!(config->flags & ~USE_LABEL))
-			d_tail = Flag2FreeStorage (&PagerDecorationSyntax, d_tail, PAGER_DECOR_NOLABEL_ID);
-		if (!(config->flags & PAGE_SEPARATOR))
-			d_tail = Flag2FreeStorage (&PagerDecorationSyntax, d_tail, PAGER_DECOR_NOSEPARATOR_ID);
-		if (!(config->flags & SHOW_SELECTION))
-			d_tail = Flag2FreeStorage (&PagerDecorationSyntax, d_tail, PAGER_DECOR_NOSELECTION_ID);
-		if (config->flags & LABEL_BELOW_DESK)
-			d_tail = Flag2FreeStorage (&PagerDecorationSyntax, d_tail, PAGER_DECOR_LABEL_BELOW_ID);
-		if (config->flags & HIDE_INACTIVE_LABEL)
-			d_tail = Flag2FreeStorage (&PagerDecorationSyntax, d_tail, PAGER_DECOR_HIDE_INACTIVE_ID);
-
-		/* selection_color */
-		d_tail = String2FreeStorage (&PagerDecorationSyntax, d_tail, config->selection_color, PAGER_DECOR_SEL_COLOR_ID);
-		/* border_width */
-		d_tail =
-			Integer2FreeStorage (&PagerDecorationSyntax, d_tail, config->border_width, PAGER_DECOR_BORDER_WIDTH_ID);
-		/* grid_color */
-		d_tail = String2FreeStorage (&PagerDecorationSyntax, d_tail, config->grid_color, PAGER_DECOR_GRID_COLOR_ID);
-		/* border_color */
-		d_tail = String2FreeStorage (&PagerDecorationSyntax, d_tail, config->border_color, PAGER_DECOR_BORDER_COLOR_ID);
-
-		Writeto (Decorto, &DecorStorage, CDT_Data, (void **)&Decorations, 0);
-		Destroyto (Decorto);
-		if (DecorStorage)
-			DestroyFreeStorage (&DecorStorage);
-
-		tail = String2FreeStorage (&PagerSyntax, tail, Decorations, PAGER_DECORATION_ID);
-		if (Decorations)
-		{
-			free (Decorations);
-			Decorations = NULL;
-		}
-	}
-
-	/* writing config into the file */
-	Writeto (PagertoWriter, &Storage, CDT_Filename, (void **)&filename, flags);
-	Destroyto (PagertoWriter);
-
-	if (Storage)
-	{
-		fprintf (stderr, "\n%s:to Writing warning: Not all Free Storage discarded! Trying again...", myname);
-		DestroyFreeStorage (&Storage);
-		fprintf (stderr, (Storage != NULL) ? " failed." : " success.");
-	}
-	return 0;
-}
-
-#endif
