@@ -208,7 +208,7 @@ void usage(void) {
 		" To display image.jpg on root window : \n"
 		"   ascompose -r -s \"<img src=image.jpg/>\""
 		" To display image.jpg on root window scaling it to screen size: \n"
-		"   ascompose -r -s \"<scale width=xroot.width height=proportional><img src=image.jpg/></scale>\"\n"
+		"   ascompose -r -s \"<scale width=$xroot.width height=proportional><img src=image.jpg/></scale>\"\n"
 	);
 }
 
@@ -274,6 +274,7 @@ Window showimage(ASImage* im, Bool looping, Window main_window, ASComposeWinProp
 Window make_main_window(Bool on_root, ASComposeWinProps *props);	
 
 int screen = 0, depth = 0;
+Display *dpy = NULL;
 
 int main(int argc, char** argv) {
 
@@ -329,19 +330,13 @@ int main(int argc, char** argv) {
 			version();
 			exit(0);
 		} else if (!strcmp(argv[i], "--quiet") || !strcmp(argv[i], "-q")) {
-#if (HAVE_AFTERBASE_FLAG==1)
 			set_output_threshold(0);
-#endif
 			verbose = 0; quiet = True ;
 		} else if (!strcmp(argv[i], "--verbose") || !strcmp(argv[i], "-V")) {
-#if (HAVE_AFTERBASE_FLAG==1)
 			set_output_threshold(OUTPUT_VERBOSE_THRESHOLD);
-#endif
 			verbose++;
 		} else if (!strcmp(argv[i], "--debug") || !strcmp(argv[i], "-D")) {
-#if (HAVE_AFTERBASE_FLAG==1)
 			set_output_threshold(OUTPUT_LEVEL_DEBUG);
-#endif
 			verbose+=2;
 		} else if ((!strcmp(argv[i], "--file") || !strcmp(argv[i], "-f")) && i < argc + 1) {
 			doc_file = argv[++i];
@@ -737,7 +732,10 @@ int main(int argc, char** argv) {
 	if (main_window_props.canvas)
 		XFreePixmap(dpy, main_window_props.canvas); 
 	if( dpy )
+	{
         XCloseDisplay (dpy);
+		dpy = NULL;
+	}
 #endif
 	LOCAL_DEBUG_OUT( "display Closed%s","");
 #ifdef DEBUG_ALLOCS
@@ -799,9 +797,12 @@ make_main_window(Bool onroot, ASComposeWinProps *props)
 	return w;
 }
 
+int asvisual_empty_XErrorHandler (Display * dpy, XErrorEvent * event);
+
 Bool
 set_root_pixmap_property(long pmap) /* Must have long type to work with XChangeProp on 64 bit machines !!!*/
 {
+#ifndef X_DISPLAY_MISSING		  
 	Window root = DefaultRootWindow(dpy);
 	char  *names[2] = {"_XROOTPMAP_ID", "ESETROOT_PMAP_ID"};
 	Atom  atoms[2];
@@ -832,10 +833,14 @@ set_root_pixmap_property(long pmap) /* Must have long type to work with XChangeP
 				if (format != 32 || nitems == 0 || pmaps[i] != pmaps[0] || type != XA_PIXMAP) 
 					break;
 			}
-		if (i>= 2 && pmaps[0]) 
+		if (i>= 2 && pmaps[0] && pmaps[0] != pmap) 
 		{
+			/* XKillClient is a dangerous affair since properties may hold stale values tha are no longer valid */
+			int (*oldXErrorHandler) (Display *, XErrorEvent *) = XSetErrorHandler (asvisual_empty_XErrorHandler);
 			LOCAL_DEBUG_OUT("killing client for pmap %X", pmaps[0]);
            	XKillClient(dpy, pmaps[0]);
+			XSync(dpy, False);
+			XSetErrorHandler (oldXErrorHandler);
 		}
     }
 	for (i = 0 ; i < 2 ; ++i)
@@ -843,18 +848,22 @@ set_root_pixmap_property(long pmap) /* Must have long type to work with XChangeP
 		if (!atoms[i])
 			if ( (atoms[i] = XInternAtom (dpy, names[i], False)) == None)
 				break;
+		LOCAL_DEBUG_OUT("Changing property %X to pmap id %X", atoms[i], pmap);
     	XChangeProperty(dpy, root, atoms[i], XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &pmap, 1);
 	}
-	if (i < 2)
-		return False;
-	
-	XSetCloseDownMode(dpy, RetainPermanent);
-	return True;
+	if (i >= 2)
+	{
+		XSetCloseDownMode(dpy, RetainPermanent);
+		return True;
+	}
+#endif
+	return False;
 }
 
 Bool
 wait_x_timeout(int timeout)
 {
+#ifndef X_DISPLAY_MISSING		  
 	if (timeout < 0) /* wait forever */
 	{
 		XEvent evt;
@@ -878,6 +887,8 @@ wait_x_timeout(int timeout)
 	}
 	
 	return XPending(dpy);
+#endif
+	return 0;	
 }
 
 Window showimage(ASImage* im, Bool looping, Window main_window, ASComposeWinProps *props, int dst_x, int dst_y ) 
@@ -1020,6 +1031,8 @@ Window showimage(ASImage* im, Bool looping, Window main_window, ASComposeWinProp
 		}		   
 	}
 
+	LOCAL_DEBUG_OUT("canvas pixmap set to %lX", props->canvas);
+
 	if (props->canvas == None)
 	{
 		int depth = 0;
@@ -1034,6 +1047,8 @@ Window showimage(ASImage* im, Bool looping, Window main_window, ASComposeWinProp
 			gc = DefaultGC(dpy, screen);
 		}
 		props->canvas = create_visual_pixmap( asv, main_window, pmap_width, pmap_height, DefaultDepth(dpy,screen));
+		LOCAL_DEBUG_OUT("Created new canvas pixmap %lX", props->canvas);
+
 		if (gc == NULL)
 			gc = XCreateGC( dpy, main_window, 0, NULL);
 		
@@ -1080,11 +1095,19 @@ Window showimage(ASImage* im, Bool looping, Window main_window, ASComposeWinProp
 	XClearWindow( dpy, main_window);
 	XFlush(dpy);
 	if (main_window == root && !looping)
-		set_root_pixmap_property(props->canvas);
-	XSync(dpy,False);
+	{
+		if (set_root_pixmap_property(props->canvas))
+		{
+			props->canvas = None;
+			XSync(dpy,False);
+		}
+	}
 		
 	if (saved_canvas)
+	{
+		LOCAL_DEBUG_OUT("Freeing saved canvas pixmap %lX", saved_canvas);
 		XFreePixmap( dpy, saved_canvas);
+	}
 
 	if (main_window == root && !looping)
 		return root;
