@@ -19,7 +19,7 @@
  *
  */
 
-#define LOCAL_DEBUG
+#undef LOCAL_DEBUG
 #ifdef _WIN32
 #include "win32/config.h"
 #else
@@ -403,15 +403,15 @@ Bool save_asimage_to_file(const char *file2bsaved, ASImage *im,
 void show_asimage(ASVisual *asv, ASImage* im, Window w, long delay)
 {
 #ifndef X_DISPLAY_MISSING
-	if ( im && w )
+	if ( im && w && asv)
 	{
 		Pixmap p = asimage2pixmap(asv, w, im, NULL, False);
 		struct timeval value;
 
-		XSetWindowBackgroundPixmap( dpy, w, p );
-		XClearWindow( dpy, w );
-		XFlush( dpy );
-		XFreePixmap( dpy, p );
+		XSetWindowBackgroundPixmap( asv->dpy, w, p );
+		XClearWindow( asv->dpy, w );
+		XFlush( asv->dpy );
+		XFreePixmap( asv->dpy, p );
 		p = None ;
 		value.tv_usec = delay % 10000;
 		value.tv_sec = delay / 10000;
@@ -518,7 +518,7 @@ translate_tag_size(	const char *width_str, const char *height_str, ASImage *imtm
  * SYNOPSIS
  * <text id="new_id" font="font" point="size" fgcolor="color"
  *       bgcolor="color" fgimage="image_id" bgimage="image_id"
- *       spacing="points">My Text Here</text>
+ *       spacing="points" type="3dtype">My Text Here</text>
  * ATTRIBUTES
  * id       Optional.  Image will be given this name for future reference.
  * font     Optional.  Default is "fixed".  Font to use for text.
@@ -1948,6 +1948,103 @@ handle_asxml_tag_slice( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* par
 	return result;
 }
 
+/****** libAfterImage/asimagexml/pixelize
+ * NAME
+ * pixelize - pixelize image using arbitrary pixel size
+ * SYNOPSIS
+ * <pixelize id="new_id" ref_id="other_imag" width="pixels" height="pixels"
+ *        clip_x="clip_x" clip_y="clip_y"
+ *        pixel_width="pixel_width" pixel_height="pixel_height">
+ * ATTRIBUTES
+ * id       Optional. Image will be given this name for future reference.
+ * refid    Optional.  An image ID defined with the "id" parameter for
+ *          any previously created image.  If set, percentages in "width"
+ *          and "height" will be derived from the width and height of the
+ *          refid image.
+ * width    Required.  The image will be scaled to this width.
+ * height   Required.  The image will be scaled to this height.
+ * clip_x   Optional. Offset into original image.
+ * clip_y   Optional. Offset into original image.
+ * pixel_width Required. Horizontal pixelization step;
+ * pixel_height Required. Vertical pixelization step;
+ * NOTES
+ * This tag applies to the first image contained within the tag.  Any
+ * further images will be discarded.
+ * If you want to keep image proportions while resizing-use "proportional"
+ * instead of specific size for particular dimention.
+ ******/
+static ASImage *
+handle_asxml_tag_pixelize( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm, ASImage *imtmp, int width, int height)
+{
+	ASImage *result = NULL ;
+	xml_elem_t* ptr;
+	int clip_x = 0, clip_y = 0 ;
+	int pixel_width = 1, pixel_height = 1 ;
+	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
+	for (ptr = parm ; ptr ; ptr = ptr->next) 
+	{
+		if (!strcmp(ptr->tag, "clip_x")) 		clip_x = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "clip_y")) 	clip_y = (int)parse_math(ptr->parm, NULL, height);
+		else if (!strcmp(ptr->tag, "pixel_width")) 		pixel_width = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "pixel_height")) 	pixel_height = (int)parse_math(ptr->parm, NULL, height);
+	}
+
+	if( state->verbose > 1 )
+		show_progress("Pixelizing image to [%dx%d] using pixel size %dx%d.", 
+						width, height, pixel_width, pixel_height);
+	result = pixelize_asimage (state->asv, imtmp, clip_x, clip_y, width, height,
+							   pixel_width, pixel_height,
+							   ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT);
+	return result;
+}
+
+/****** libAfterImage/asimagexml/color2alpha
+ * NAME
+ * color2alpha - set alpha channel based on color closeness to specified color
+ * SYNOPSIS
+ * <color2alpha id="new_id" ref_id="other_imag" width="pixels" height="pixels"
+ *        clip_x="clip_x" clip_y="clip_y"
+ *        color="color">
+ * ATTRIBUTES
+ * id       Optional. Image will be given this name for future reference.
+ * refid    Optional.  An image ID defined with the "id" parameter for
+ *          any previously created image.  If set, percentages in "width"
+ *          and "height" will be derived from the width and height of the
+ *          refid image.
+ * width    Required.  The image will be scaled to this width.
+ * height   Required.  The image will be scaled to this height.
+ * clip_x   Optional. Offset into original image.
+ * clip_y   Optional. Offset into original image.
+ * color    Required. Color to match against.
+ * NOTES
+ * This tag applies to the first image contained within the tag.  Any
+ * further images will be discarded.
+ * If you want to keep image proportions while resizing-use "proportional"
+ * instead of specific size for particular dimention.
+ ******/
+static ASImage *
+handle_asxml_tag_color2alpha( ASImageXMLState *state, xml_elem_t* doc, xml_elem_t* parm, ASImage *imtmp, int width, int height)
+{
+	ASImage *result = NULL ;
+	xml_elem_t* ptr;
+	int clip_x = 0, clip_y = 0 ;
+	ARGB32 color;
+	LOCAL_DEBUG_OUT("doc = %p, parm = %p, imtmp = %p, width = %d, height = %d", doc, parm, imtmp, width, height ); 
+	for (ptr = parm ; ptr ; ptr = ptr->next) 
+	{
+		if (!strcmp(ptr->tag, "clip_x")) 		clip_x = (int)parse_math(ptr->parm, NULL, width);
+		else if (!strcmp(ptr->tag, "clip_y")) 	clip_y = (int)parse_math(ptr->parm, NULL, height);
+		else if (!strcmp(ptr->tag, "color")) 	parse_argb_color(ptr->parm, &color);
+	}
+
+	if( state->verbose > 1 )
+		show_progress("color2alpha image to [%dx%d] using color #%8.8X.", width, height, color);
+	result = color2alpha_asimage (state->asv, imtmp, clip_x, clip_y, width, height,
+							   		color,
+							   		ASA_ASImage, 100, ASIMAGE_QUALITY_DEFAULT);
+	return result;
+}
+
 /****** libAfterImage/asimagexml/crop
  * NAME
  * crop - crop image to arbitrary area within it.
@@ -2265,6 +2362,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 
 		if( refid ) 
 			refimg = fetch_asimage( imman, refid);
+
 		if (!strcmp(doc->tag, "composite")) 
 			result = handle_asxml_tag_composite( &state, doc, parm );  	
 		else if (!strcmp(doc->tag, "text")) 
@@ -2316,24 +2414,21 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		
 					if ( width > 0 && height > 0 )
 					{ 
-						if( !strcmp(doc->tag, "bevel") )
-							result = handle_asxml_tag_bevel( &state, doc, parm, imtmp, width, height ); 	   
-						else if( !strcmp(doc->tag, "mirror") )
-							result = handle_asxml_tag_mirror( &state, doc, parm, imtmp, width, height);
-						else if ( !strcmp(doc->tag, "rotate"))
-					   		result = handle_asxml_tag_rotate( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "scale"))
-							result = handle_asxml_tag_scale( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "slice"))
-							result = handle_asxml_tag_slice( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "crop"))
-							result = handle_asxml_tag_crop( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "tile"))
-							result = handle_asxml_tag_tile( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "hsv"))
-							result = handle_asxml_tag_hsv( &state, doc, parm, imtmp, width, height);
-						else if (!strcmp(doc->tag, "pad"))
-							result = handle_asxml_tag_pad( &state, doc, parm, imtmp, width, height);
+#define HANDLE_SIZED_TAG(ttag) \
+		else if( !strcmp(doc->tag, #ttag) )	result = handle_asxml_tag_##ttag( &state, doc, parm, imtmp, width, height )
+						if (0){}
+						HANDLE_SIZED_TAG(bevel);
+						HANDLE_SIZED_TAG(mirror);
+						HANDLE_SIZED_TAG(rotate);
+						HANDLE_SIZED_TAG(scale);
+						HANDLE_SIZED_TAG(slice);
+						HANDLE_SIZED_TAG(crop);
+						HANDLE_SIZED_TAG(tile);
+						HANDLE_SIZED_TAG(hsv);
+						HANDLE_SIZED_TAG(pad);
+						HANDLE_SIZED_TAG(pixelize);
+						HANDLE_SIZED_TAG(color2alpha);
+#undef HANDLE_SIZED_TAG						
 					}		
 				}
 				
