@@ -41,6 +41,7 @@
 # include "afterbase.h"
 #endif
 #include "asvisual.h"
+#include "scanline.h"
 
 #if defined(XSHMIMAGE) && !defined(X_DISPLAY_MISSING) 
 # include <sys/ipc.h>
@@ -131,68 +132,6 @@ int get_bits_per_pixel(Display *dpy, int depth)
 	return 32;
  }
 
-/* ********************* ASScanline ************************************/
-ASScanline*
-prepare_scanline( unsigned int width, unsigned int shift, ASScanline *reusable_memory, Bool BGR_mode  )
-{
-	register ASScanline *sl = reusable_memory ;
-	size_t aligned_width;
-	void *ptr;
-
-	if( sl == NULL )
-		sl = safecalloc( 1, sizeof( ASScanline ) );
-	else
-		memset( sl, 0x00, sizeof(ASScanline));
-
-	if( width == 0 ) width = 1 ;
-	sl->width 	= width ;
-	sl->shift   = shift ;
-	/* we want to align data by 8 byte boundary (double)
-	 * to allow for code with less ifs and easier MMX/3Dnow utilization :*/
-	aligned_width = width + (width&0x00000001);
-	sl->buffer = ptr = safemalloc (((aligned_width*4)+16)*sizeof(CARD32)+8);
-
-	sl->xc1 = sl->red 	= (CARD32*)((((long)ptr+7)>>3)*8);
-	sl->xc2 = sl->green = sl->red   + aligned_width;
-	sl->xc3 = sl->blue 	= sl->green + aligned_width;
-	sl->alpha 	= sl->blue  + aligned_width;
-
-	sl->channels[IC_RED] = sl->red ;
-	sl->channels[IC_GREEN] = sl->green ;
-	sl->channels[IC_BLUE] = sl->blue ;
-	sl->channels[IC_ALPHA] = sl->alpha ;
-
-	if( BGR_mode )
-	{
-		sl->xc1 = sl->blue ;
-		sl->xc3 = sl->red ;
-	}
-	/* this way we can be sure that our buffers have size of multiplies of 8s
-	 * and thus we can skip unneeded checks in code */
-	/* initializing padding into 0 to avoid any garbadge carry-over
-	 * bugs with diffusion: */
-	sl->red[aligned_width-1]   = 0;
-	sl->green[aligned_width-1] = 0;
-	sl->blue[aligned_width-1]  = 0;
-	sl->alpha[aligned_width-1] = 0;
-	
-	sl->back_color = ARGB32_DEFAULT_BACK_COLOR;
-
-	return sl;
-}
-
-void
-free_scanline( ASScanline *sl, Bool reusable )
-{
-	if( sl )
-	{
-		if( sl->buffer )
-			free( sl->buffer );
-		if( !reusable )
-			free( sl );
-	}
-}
-
 /* ********************* ASVisual ************************************/
 ASVisual *_set_default_asvisual( ASVisual *new_v );
 
@@ -465,9 +404,9 @@ destroy_asvisual( ASVisual *asv, Bool reusable )
 		}
 #ifdef HAVE_GLX
 		if( asv->glx_scratch_gc_direct )
-			glXDestroyContext(dpy, asv->glx_scratch_gc_direct );
+			glXDestroyContext(asv->dpy, asv->glx_scratch_gc_direct );
 		if( asv->glx_scratch_gc_indirect )
-			glXDestroyContext(dpy, asv->glx_scratch_gc_indirect );
+			glXDestroyContext(asv->dpy, asv->glx_scratch_gc_indirect );
 #endif
 		if( asv->scratch_window ) 
 			XDestroyWindow( asv->dpy, asv->scratch_window );
@@ -1402,10 +1341,10 @@ destroy_xshmimage_image(ASHashableValue value, void *data)
 	}
 }
 
-Bool enable_shmem_images ( )
+Bool enable_shmem_images_for_visual (ASVisual *asv)
 {
 #ifndef DEBUG_ALLOCS
-	if( XShmQueryExtension (dpy) )
+	if( asv && asv->dpy && XShmQueryExtension (asv->dpy) )
 	{
 		_as_use_shm_images = True ;
 		if( xshmimage_segments == NULL )
@@ -1418,6 +1357,12 @@ Bool enable_shmem_images ( )
 	return _as_use_shm_images;
 }
 
+Bool enable_shmem_images ()
+{
+	return enable_shmem_images_for_visual (get_default_asvisual());
+}
+
+
 void disable_shmem_images()
 {
 	_as_use_shm_images = False ;
