@@ -12,13 +12,9 @@ extern "C" {
  * SEE ALSO
  * Structures:
  *  	    ColorPair
- *  	    ASScanline
  *  	    ASVisual
  *
  * Functions :
- *   ASScanline handling:
- *  	    prepare_scanline(), free_scanline()
- *
  *   ASVisual initialization :
  *  	    query_screen_visual(), setup_truecolor_visual(),
  *  	    setup_pseudo_visual(), setup_as_colormap(),create_asvisual(),
@@ -90,13 +86,26 @@ typedef CARD32 ARGB32;
 #define ARGB32_CHAN8(c,i)		(((c)>>((i)<<3))&0x00FF)
 #define MAKE_ARGB32_CHAN8(v,i)	(((v)&0x0000FF)<<((i)<<3))
 
-#define ARGB32_ALPHA16(c)		((((c)>>16)&0x00FF00)|0x00FF)
-#define ARGB32_RED16(c)			((((c)>>8)&0x00FF00)|0x00FF)
-#define ARGB32_GREEN16(c)	 	(( (c)    &0x00FF00)|0x00FF)
-#define ARGB32_BLUE16(c)		((((c)<<8)&0x00FF00)|0x00FF)
-#define ARGB32_CHAN16(c,i)		((ARGB32_CHAN8(c,i)<<8)|0x00FF)
+#ifdef __GNUC__
+#define ARGB32_ALPHA16(c)		({ CARD32 __c = ARGB32_ALPHA8(c); __c | (__c<<8);})
+#define ARGB32_RED16(c)			({ CARD32 __c = ARGB32_RED8(c); __c | (__c<<8);})
+#define ARGB32_GREEN16(c)	 	({ CARD32 __c = ARGB32_GREEN8(c); __c | (__c<<8);})
+#define ARGB32_BLUE16(c)		({ CARD32 __c = ARGB32_BLUE8(c); __c | (__c<<8);})
+#define ARGB32_CHAN16(c,i)		({ CARD32 __c = ARGB32_CHAN8(c,i); __c | (__c<<8);})
+#else
+#define ARGB32_ALPHA16(c)		((((c)>>16)&0x00FF00)|(((c)>>24)&0x0000FF))
+#define ARGB32_RED16(c)			((((c)>>8 )&0x00FF00)|(((c)>>16)&0x0000FF))
+#define ARGB32_GREEN16(c)	 	(( (c)     &0x00FF00)|(((c)>>8 )&0x0000FF))
+#define ARGB32_BLUE16(c)		((((c)<<8) &0x00FF00)|(((c)    )&0x0000FF))
+#define ARGB32_CHAN16(c,i)		((ARGB32_CHAN8(c,i)<<8)|ARGB32_CHAN8(c,i))
+#endif
+
 #define MAKE_ARGB32_CHAN16(v,i)	((((v)&0x00FF00)>>8)<<((i)<<3))
 /*******************/
+
+
+struct ASScanline;
+
 /****d* libAfterImage/ColorPart
  * NAME
  * IC_RED - red channel
@@ -134,53 +143,6 @@ typedef struct ColorPair
   ARGB32 back;
 }ColorPair;
 /*******************/
-/****s* libAfterImage/ASScanline
- * NAME
- * ASScanline - structure to hold contents of the single scanline.
- * DESCRIPTION
- * ASScanline holds data for the single scanline, split into channels
- * with 32 bits per pixel per channel. All the memory is allocated at
- * once, and then split in between channels. There are three ways to
- * access channel data :
- * 1) using blue, green, red, alpha pointers.
- * 2) using channels[] array of pointers - convenient in loops
- * 4) using xc3, xc2, xc1 pointers. These are different from red, green,
- * blue in the way that xc3 will point to blue when BGR mode is specified
- * at the time of creation, otherwise it will point to red channel.
- * Likewise xc1 will point to red in BGR mode and blue otherwise.
- * xc2 always points to green channel's data. This is convenient while
- * writing XImages and when channels in source and destination has to be
- * reversed, while reading images from files.
- * Channel data is always aligned by 8 byte boundary allowing for
- * utilization of MMX, floating point and other 64bit registers for
- * transfer and processing.
- * SEE ALSO
- * ASImage
- * SOURCE
- */
-typedef struct ASScanline
-{
-#define SCL_DO_BLUE         (0x01<<ARGB32_BLUE_CHAN )
-#define SCL_DO_GREEN        (0x01<<ARGB32_GREEN_CHAN)
-#define SCL_DO_RED          (0x01<<ARGB32_RED_CHAN  )
-#define SCL_DO_ALPHA		(0x01<<ARGB32_ALPHA_CHAN)
-#define SCL_DO_COLOR		(SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE)
-#define SCL_DO_ALL			(SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE| \
-                             SCL_DO_ALPHA)
-	CARD32	 	   flags ;   /* combination of  the above values */
-	CARD32        *buffer ;
-	CARD32        *blue, *green, *red, *alpha ;
-	CARD32	      *channels[IC_NUM_CHANNELS];
-	CARD32        *xc3, *xc2, *xc1; /* since some servers require
-									 * BGR mode here we store what
-									 * goes into what color component
-									 * in XImage */
-	ARGB32         back_color;
-	unsigned int   width, shift;
-	unsigned int   offset_x ;
-}ASScanline;
-/*******************/
-
 /****f* libAfterImage/ARGB32_manhattan_distance()
  * NAME 
  * ARGB32_manhattan_distance() - This function can be used to evaluate closeness of 
@@ -194,46 +156,6 @@ typedef struct ASScanline
  *********/
 long ARGB32_manhattan_distance (long a, long b);
 
-/****f* libAfterImage/prepare_scanline()
- * NAME
- * prepare_scanline()
- * SYNOPSIS
- * ASScanline *prepare_scanline ( unsigned int width,
- *                                unsigned int shift,
- *                                ASScanline *reusable_memory,
- *                                Bool BGR_mode);
- * INPUTS
- * width           - width of the scanline.
- * shift           - format of contained data. 0 means - 32bit unshifted
- *                   8 means - 24.8bit ( 8 bit left shifted ).
- * reusable_memory - preallocated object.
- * BGR_mode        - if True will cause xc3 to point to Blue and xc1 to
- *                   point to red.
- * DESCRIPTION
- * This function allocates memory ( if reusable_memory is NULL ) for
- * the new ASScanline structure. Structures buffers gets allocated to
- * hold scanline data of at least width pixel wide. Buffers are adjusted
- * to start on 8 byte boundary.
- *********/
-/****f* libAfterImage/free_scanline()
- * NAME
- * free_scanline()
- * SYNOPSIS
- * void       free_scanline ( ASScanline *sl, Bool reusable );
- * INPUTS
- * sl       - pointer to previously allocated ASScanline structure to be
- *            deallocated.
- * reusable - if true then ASScanline object itself will not be
- *            deallocated.
- * DESCRIPTION
- * free_scanline() frees all the buffer memory allocated for ASScanline.
- * If reusable is false then object itself in not freed. That is usable
- * for declaring ASScanline on stack.
- *********/
-ASScanline* prepare_scanline( unsigned int width, unsigned int shift,
-	                          ASScanline *reusable_memory, Bool BGR_mode);
-void       free_scanline( ASScanline *sl, Bool reusable );
-
 /****s* libAfterImage/ASVisual
  * NAME
  * ASVisual - an abstraction layer on top of X Server Visual.
@@ -330,11 +252,11 @@ typedef struct ASVisual
 									CARD32 *blue);
 	void   (*ximage2scanline_func)( struct ASVisual *asv, 
 									XImage *xim,
-		                            ASScanline *sl, int y,
+		                            struct ASScanline *sl, int y,
 								    unsigned char *xim_data );
 	void   (*scanline2ximage_func)( struct ASVisual *asv, 
 									XImage *xim,
-									ASScanline *sl, int y,
+									struct ASScanline *sl, int y,
 									unsigned char *xim_data );
 
 #define ASGLX_Unavailable			0
@@ -366,6 +288,40 @@ typedef struct ASVisual
 #endif
 }ASVisual;
 /*******************/
+CARD32 color2pixel32bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel32rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel24bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel24rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel16bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel16rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel15bgr(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel15rgb(ASVisual *asv, CARD32 encoded_color, unsigned long *pixel);
+CARD32 color2pixel_pseudo3bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
+CARD32 color2pixel_pseudo6bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
+CARD32 color2pixel_pseudo12bpp( ASVisual *asv, CARD32 encoded_color, unsigned long *pixel );
+
+void pixel2color32rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color32bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color24rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color24bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color16rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color16bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color15rgb(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *green, CARD32 *blue);
+
+void ximage2scanline32( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void ximage2scanline16( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void ximage2scanline15( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void ximage2scanline_pseudo3bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void ximage2scanline_pseudo6bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+
+void scanline2ximage32( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void scanline2ximage16( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void scanline2ximage15( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void scanline2ximage_pseudo3bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void scanline2ximage_pseudo6bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
+void scanline2ximage_pseudo12bpp( ASVisual *asv, XImage *xim, struct ASScanline *sl, int y,  register unsigned char *xim_data );
 
 /****f* libAfterImage/query_screen_visual()
  * NAME
@@ -685,6 +641,10 @@ Pixmap  create_visual_pixmap( ASVisual *asv, Window root,
 							  unsigned int depth );
 void destroy_visual_pixmap( ASVisual *asv, Pixmap *ppmap );
 
+int get_dpy_drawable_size (Display *drawable_dpy, Drawable d, unsigned int *ret_w, unsigned int *ret_h);
+Bool get_dpy_window_position (Display *window_dpy, Window root, Window w, int *px, int *py, int *transparency_x, int *transparency_y);
+
+
 XImage* create_visual_ximage( ASVisual *asv,
 	                          unsigned int width, unsigned int height,
 							  unsigned int depth );
