@@ -97,9 +97,21 @@ Atom          _XA_NET_WM_STATE_HIDDEN;
 Atom          _XA_NET_WM_STATE_FULLSCREEN;
 Atom          _XA_NET_WM_STATE_ABOVE;
 Atom          _XA_NET_WM_STATE_BELOW;
+Atom          _XA_NET_WM_STATE_DEMANDS_ATTENTION;
 
 Atom          _XA_NET_WM_PID;
+Atom          _XA_NET_WM_ICON;
 Atom          _XA_NET_WM_PING;
+Atom 		  _XA_NET_WM_WINDOW_OPACITY;
+
+/* Implements KDE System tray specs:
+ * http://developer.kde.org/documentation/library/kdeqt/kde3arch/protocols-docking.html
+ * https://listman.redhat.com/archives/xdg-list/2002-March/msg00014.html
+ * http://standards.freedesktop.org/systemtray-spec/systemtray-spec-0.2.html#ftn.id2494129
+ * 
+ */
+Atom          _XA_KDE_DESKTOP_WINDOW = None ;
+Atom 		  _XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR = None;
 
 
 /* Crossreferences of atoms into flag value for
@@ -122,6 +134,10 @@ AtomXref      MainHints[] = {
 	{"_NET_WM_WINDOW_TYPE", &_XA_NET_WM_WINDOW_TYPE},
 	{"_NET_WM_STATE", &_XA_NET_WM_STATE},
 	{"_NET_WM_PID", &_XA_NET_WM_PID},
+	{"_NET_WM_ICON", &_XA_NET_WM_ICON},
+	{"_NET_WM_WINDOW_OPACITY", &_XA_NET_WM_WINDOW_OPACITY},
+	{"_KDE_DESKTOP_WINDOW", &_XA_KDE_DESKTOP_WINDOW},
+	{"_KDE_NET_SYSTEM_TRAY_WINDOW_FOR", &_XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR},
 	{NULL, NULL, 0, None}
 };
 
@@ -152,11 +168,13 @@ AtomXref      _EXTWM_State[] = {
 	{"_NET_WM_STATE_MAXIMIZED_HORZ", &_XA_NET_WM_STATE_MAXIMIZED_HORZ, EXTWM_StateMaximizedH},
 	{"_NET_WM_STATE_SHADED", &_XA_NET_WM_STATE_SHADED, EXTWM_StateShaded},
     {"_NET_WM_STATE_SKIP_TASKBAR", &_XA_NET_WM_STATE_SKIP_TASKBAR, EXTWM_StateSkipTaskbar},
-	{"_NET_WM_STATE_SKIP_PAGER", &_XA_NET_WM_STATE_SKIP_TASKBAR, EXTWM_StateSkipPager},
+	{"_NET_WM_STATE_SKIP_PAGER", &_XA_NET_WM_STATE_SKIP_PAGER, EXTWM_StateSkipPager},
 	{"_NET_WM_STATE_HIDDEN", &_XA_NET_WM_STATE_HIDDEN,			EXTWM_StateHidden    },
 	{"_NET_WM_STATE_FULLSCREEN", &_XA_NET_WM_STATE_FULLSCREEN, 	EXTWM_StateFullscreen},
 	{"_NET_WM_STATE_ABOVE", &_XA_NET_WM_STATE_ABOVE, 			EXTWM_StateAbove	 },
 	{"_NET_WM_STATE_BELOW", &_XA_NET_WM_STATE_BELOW, 			EXTWM_StateBelow	 },
+	{"_NET_WM_STATE_DEMANDS_ATTENTION", 
+							&_XA_NET_WM_STATE_DEMANDS_ATTENTION,EXTWM_StateDemandsAttention	 },
 	{NULL, NULL, 0, None}
 };
 
@@ -390,7 +408,11 @@ read_wm_protocols (ASRawHints * hints, Window w)
 		hints->wm_protocols = 0;
 		if (read_32bit_proplist (w, _XA_WM_PROTOCOLS, 3, &protocols, &nprotos))
 		{
-            LOCAL_DEBUG_OUT( "nprotos=%ld, protocols[0] = 0x%lX(\"%s\")", nprotos, protocols[0], XGetAtomName(dpy, protocols[0]));
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)		
+			char *aname = XGetAtomName(dpy, protocols[0]);
+            LOCAL_DEBUG_OUT( "nprotos=%ld, protocols[0] = 0x%lX(\"%s\")", nprotos, protocols[0], aname);
+			XFree( aname );
+#endif			
 			translate_atom_list (&(hints->wm_protocols), WM_Protocols, protocols, nprotos);
             LOCAL_DEBUG_OUT( "translated protocols =0x%lX", hints->wm_protocols );
 			translate_atom_list (&(hints->extwm_hints.flags), EXTWM_Protocols, protocols, nprotos);
@@ -598,7 +620,8 @@ read_extwm_window_type (ASRawHints * hints, Window w)
 
 		if (read_32bit_proplist (w, _XA_NET_WM_WINDOW_TYPE, 6, &protocols, &nprotos))
 		{
-			translate_atom_list (&(hints->extwm_hints.flags), EXTWM_WindowType, protocols, nprotos);
+			translate_atom_list (&(hints->extwm_hints.type_flags), EXTWM_WindowType, protocols, nprotos);
+			set_flags( hints->extwm_hints.flags, EXTWM_TypeSet );
 			free (protocols);
 		}
 	}
@@ -614,7 +637,8 @@ read_extwm_state (ASRawHints * hints, Window w)
 
         if (read_32bit_proplist (w, _XA_NET_WM_STATE, 6, &protocols, &nprotos))
 		{
-			translate_atom_list (&(hints->extwm_hints.flags), EXTWM_State, protocols, nprotos);
+			translate_atom_list (&(hints->extwm_hints.state_flags), EXTWM_State, protocols, nprotos);
+			set_flags( hints->extwm_hints.flags, EXTWM_StateSet );
 			free (protocols);
 		}
 	}
@@ -631,6 +655,47 @@ read_extwm_pid (ASRawHints * hints, Window w)
 	}
 }
 
+void
+read_extwm_icon (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		if (read_32bit_proplist (w, _XA_NET_WM_ICON, 2+48*48, &(hints->extwm_hints.icon), &(hints->extwm_hints.icon_length)) )
+			set_flags (hints->extwm_hints.flags, EXTWM_ICON);
+	}
+}
+
+void
+read_extwm_window_opacity (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		if (read_32bit_property (w, _XA_NET_WM_WINDOW_OPACITY, &(hints->extwm_hints.window_opacity)))
+			set_flags (hints->extwm_hints.flags, EXTWM_WINDOW_OPACITY);
+	}
+}
+
+void
+read_kde_desktop_window (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		CARD32 dummy ;
+		if (read_32bit_property (w, _XA_KDE_DESKTOP_WINDOW, &dummy))
+			set_flags( hints->kde_hints.flags, KDE_DesktopWindow);
+	}
+}
+	
+void
+read_kde_systray_window_for (ASRawHints * hints, Window w)
+{
+	if (hints && w != None)
+	{
+		if (read_32bit_property (w, _XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR, &hints->kde_hints.systray_window_for))
+			set_flags( hints->kde_hints.flags, KDE_SysTrayWindowFor);
+	}
+}
+
 Bool
 default_parent_hints_func (Window parent, ASParentHints * dst)
 {
@@ -717,7 +782,12 @@ HintsDescriptions[] =
 	{&_XA_NET_WM_DESKTOP, read_extwm_desktop, HINT_STARTUP | HINT_PROTOCOL, HINTS_ExtendedWM},
 	{&_XA_NET_WM_WINDOW_TYPE, read_extwm_window_type, HINT_STARTUP | HINT_GENERAL, HINTS_ExtendedWM},
 	{&_XA_NET_WM_STATE, read_extwm_state, HINT_STARTUP | HINT_GENERAL, HINTS_ExtendedWM},
-	{&_XA_NET_WM_PID, read_extwm_pid, HINT_STARTUP, HINTS_ExtendedWM},
+	{&_XA_NET_WM_PID, read_extwm_pid, HINT_GENERAL, HINTS_ExtendedWM},
+	{&_XA_NET_WM_ICON, read_extwm_icon, HINT_GENERAL, HINTS_ExtendedWM},
+	{&_XA_NET_WM_WINDOW_OPACITY, read_extwm_window_opacity, HINT_GENERAL, HINTS_ExtendedWM},
+	{&_XA_KDE_DESKTOP_WINDOW, read_kde_desktop_window, HINT_GENERAL, HINTS_KDE},
+	{&_XA_KDE_NET_SYSTEM_TRAY_WINDOW_FOR, read_kde_systray_window_for, HINT_GENERAL, HINTS_KDE},
+
 	{
 	NULL, NULL, 0, HINTS_Supported}
 };
@@ -863,6 +933,8 @@ destroy_raw_hints (ASRawHints * hints, Bool reusable)
 			free_text_property (&(hints->extwm_hints.name));
 		if (hints->extwm_hints.icon_name)
 			free_text_property (&(hints->extwm_hints.icon_name));
+		if (hints->extwm_hints.icon)
+			free (hints->extwm_hints.icon);
 
 		if (reusable)						   /* we are being paranoid */
 			memset (hints, 0x00, sizeof (ASRawHints));
@@ -1000,9 +1072,15 @@ print_extwm_hints (stream_func func, void *stream, ExtendedWMHints * hints)
 		func (stream, "EXTWM._NET_WM_DESKTOP = %ld;\n", hints->desktop);
 	if (get_flags (hints->flags, EXTWM_PID))
 		func (stream, "EXTWM._NET_WM_PID = %ld;\n", hints->pid);
+	if (get_flags (hints->flags, EXTWM_ICON))
+		func (stream, "EXTWM._NET_WM_ICON.length = %ld;\n", hints->icon_length);
+	if (get_flags (hints->flags, EXTWM_WINDOW_OPACITY))
+		func (stream, "EXTWM._NET_WM_WINDOW_OPACITY = %ld;\n", hints->window_opacity);
 
-	print_list_hints (func, stream, hints->flags, EXTWM_WindowType, "EXTWM._NET_WM_WINDOW_TYPE");
-	print_list_hints (func, stream, hints->flags, EXTWM_State, "EXTWM._NET_WM_STATE");
+	if (get_flags (hints->flags, EXTWM_TypeSet))
+		print_list_hints (func, stream, hints->type_flags, EXTWM_WindowType, "EXTWM._NET_WM_WINDOW_TYPE");
+	if (get_flags (hints->flags, EXTWM_StateSet))
+		print_list_hints (func, stream, hints->state_flags, EXTWM_State, "EXTWM._NET_WM_STATE");
 	print_list_hints (func, stream, hints->flags, EXTWM_Protocols, "EXTWM._NET_WM_PROTOCOLS");
 }
 
@@ -1109,9 +1187,7 @@ handle_manager_property_update (Window w, Atom property, ASRawHints * raw)
 	if (w && property && raw)
 	{
 		/* Here we are only interested in properties updtaed by the Window Manager : */
-		if (property == _XA_WM_NAME || property == _XA_WM_ICON_NAME ||
-			property == _XA_NET_WM_NAME || property == _XA_NET_WM_ICON_NAME ||
-			property == _XA_NET_WM_VISIBLE_NAME || property == _XA_NET_WM_VISIBLE_ICON_NAME)
+		if (IsNameProp(property))
 		{
 			read_wm_state (raw, w);
 			read_wm_icon_name (raw, w);
@@ -1156,8 +1232,9 @@ get_extwm_state_flags (Window w, ASFlagType *flags)
 		CARD32         *protocols;
 		long          nprotos = 0;
 
-        if (read_32bit_proplist (w, _XA_NET_WM_STATE, 6, &protocols, &nprotos))
+        if (read_32bit_proplist (w, _XA_NET_WM_STATE, MAX_NET_WM_STATES, &protocols, &nprotos))
 		{
+/*			LOCAL_DEBUG_OUT( "natoms =  %ld", nprotos ); */
 			translate_atom_list (flags, EXTWM_State, protocols, nprotos);
 			free (protocols);
 			return True;
@@ -1181,6 +1258,7 @@ get_extwm_state_flags (Window w, ASFlagType *flags)
 void
 set_client_state (Window w, struct ASStatusHints *status)
 {
+	LOCAL_DEBUG_CALLER_OUT( "w = %lX, status->flags = 0x%lX", w, status?status->flags:0);
 	if (w != None)
 	{
         if (status != NULL)
@@ -1188,8 +1266,6 @@ set_client_state (Window w, struct ASStatusHints *status)
             CARD32        extwm_states[MAX_NET_WM_STATES];
 			long          used = 0;
 			CARD32        gnome_state = 0;
-			CARD32       *old_extwm_state = NULL;
-			long          old_extwm_used = 0;
 			ASFlagType    old_state = 0;
 
 			if (get_flags (status->flags, AS_Sticky))
@@ -1217,20 +1293,37 @@ set_client_state (Window w, struct ASStatusHints *status)
 				extwm_states[used++] = _XA_NET_WM_STATE_MAXIMIZED_VERT;
 				gnome_state |= WIN_STATE_MAXIMIZED_VERT;
 			}
-			if (read_32bit_proplist (w, _XA_NET_WM_WINDOW_TYPE, MAX_NET_WM_STATES, &old_extwm_state, &old_extwm_used))
+/*			if (get_flags (status->flags, AS_Urgent))
 			{
-				translate_atom_list (&old_state, EXTWM_State, old_extwm_state, old_extwm_used);
-				free (old_extwm_state);
+				extwm_states[used++] = _XA_NET_WM_STATE_DEMANDS_ATTENTION;
 			}
-			if (get_flags (old_state, EXTWM_StateModal))
-				extwm_states[used++] = _XA_NET_WM_STATE_MODAL;
-			if (get_flags (old_state, EXTWM_StateSkipTaskbar))
-				extwm_states[used++] = _XA_NET_WM_STATE_SKIP_TASKBAR;
-			if (get_flags (old_state, EXTWM_StateSkipPager))
-				extwm_states[used++] = _XA_NET_WM_STATE_SKIP_PAGER;
+ */			
 
-			set_32bit_proplist (w, _XA_NET_WM_STATE, XA_ATOM, &(extwm_states[0]), used);
-			set_32bit_property (w, _XA_WIN_STATE, XA_CARDINAL, gnome_state);
+/*			LOCAL_DEBUG_OUT( "window %lX used =  %ld", w, used ); */
+			if( get_extwm_state_flags (w, &old_state) )
+			{
+				if (get_flags (old_state, EXTWM_StateModal))
+					extwm_states[used++] = _XA_NET_WM_STATE_MODAL;
+				if (get_flags (old_state, EXTWM_StateSkipTaskbar))
+					extwm_states[used++] = _XA_NET_WM_STATE_SKIP_TASKBAR;
+				if (get_flags (old_state, EXTWM_StateSkipPager))
+					extwm_states[used++] = _XA_NET_WM_STATE_SKIP_PAGER;
+				if (get_flags (old_state, EXTWM_StateDemandsAttention))
+					extwm_states[used++] = _XA_NET_WM_STATE_DEMANDS_ATTENTION;
+				if (get_flags (old_state, EXTWM_StateHidden))
+					extwm_states[used++] = _XA_NET_WM_STATE_HIDDEN;
+			}
+			LOCAL_DEBUG_OUT( "window %lX old_extwm_state = 0x%lX, used =  %ld", w, old_state, used );
+
+			if( used == 0 )
+			{
+				XDeleteProperty( dpy, w, _XA_NET_WM_STATE );
+				XDeleteProperty( dpy, w, _XA_WIN_STATE );
+			}else
+			{
+				set_32bit_proplist (w, _XA_NET_WM_STATE, XA_ATOM, &(extwm_states[0]), used);
+				set_32bit_property (w, _XA_WIN_STATE, XA_CARDINAL, gnome_state);
+			}
 
 			if (get_flags (status->flags, AS_Layer))
 				set_32bit_property (w, _XA_WIN_LAYER, XA_CARDINAL, status->layer);
@@ -1238,6 +1331,47 @@ set_client_state (Window w, struct ASStatusHints *status)
     }
 }
 
+void
+set_extwm_urgency_state (Window w, Bool set )
+{
+	LOCAL_DEBUG_CALLER_OUT( "w = %lX, set = %d", w, set);
+	if (w != None)
+	{
+		CARD32         *states = NULL;
+		long          nstates = 0;
+		int i ; 
+		Bool changed = False ;
+
+		read_32bit_proplist (w, _XA_NET_WM_STATE, MAX_NET_WM_STATES, &states, &nstates);
+		
+		for( i = 0 ; i < nstates ; ++i ) 
+			if( states[i] == _XA_NET_WM_STATE_DEMANDS_ATTENTION ) 
+				break ;
+		if( set && i >= nstates ) 
+		{
+			++nstates ; 
+			states = realloc( states, sizeof(CARD32)*nstates );
+			states[nstates-1] = _XA_NET_WM_STATE_DEMANDS_ATTENTION ;
+			changed = True ;
+		}else if( !set && i < nstates ) 
+		{
+			while( ++i < nstates )	states[i-1] = states[i];
+			--nstates ; 
+			changed = True ;
+		}
+		if( changed ) 
+		{
+			if( nstates == 0 )
+				XDeleteProperty( dpy, w, _XA_NET_WM_STATE );
+			else
+				set_32bit_proplist (w, _XA_NET_WM_STATE, XA_ATOM, &states[0], nstates);
+		}
+		free( states );
+    }
+}
+
+
+
 void
 set_client_desktop (Window w, int ext_desk)
 {
@@ -1289,7 +1423,7 @@ set_client_protocols (Window w, ASFlagType protocols, ASFlagType extwm_protocols
 LOCAL_DEBUG_OUT( "protocols=0x%lX", protocols );
     if (w && protocols)
 	{
-		CARD32         *list, *extwm_list;
+		CARD32         *list = NULL, *extwm_list = NULL;
 		long          nitems, extwm_nitems;
 
 		encode_atom_list (&(WM_Protocols[0]), &list, &nitems, protocols);
@@ -1321,23 +1455,33 @@ set_extwm_hints (Window w, ExtendedWMHints * extwm_hints)
 		CARD32         *list;
 		long          nitems;
 
-		encode_atom_list (&(EXTWM_WindowType[0]), &list, &nitems, extwm_hints->flags);
-		if (nitems > 0)
+		if (get_flags (extwm_hints->flags, EXTWM_TypeSet))
 		{
-			set_32bit_proplist (w, _XA_NET_WM_WINDOW_TYPE, XA_ATOM, list, nitems);
-			free (list);
+			encode_atom_list (&(EXTWM_WindowType[0]), &list, &nitems, extwm_hints->type_flags);
+			if (nitems > 0)
+			{
+				set_32bit_proplist (w, _XA_NET_WM_WINDOW_TYPE, XA_ATOM, list, nitems);
+				free (list);
+			}
 		}
-		encode_atom_list (&(EXTWM_State[0]), &list, &nitems, extwm_hints->flags);
-		if (nitems > 0)
+		if (get_flags (extwm_hints->flags, EXTWM_StateSet))
 		{
-			set_32bit_proplist (w, _XA_NET_WM_STATE, XA_CARDINAL, list, nitems);
-			free (list);
-            list = NULL ;
+			encode_atom_list (&(EXTWM_State[0]), &list, &nitems, extwm_hints->state_flags);
+			if (nitems > 0)
+			{
+				set_32bit_proplist (w, _XA_NET_WM_STATE, XA_CARDINAL, list, nitems);
+				free (list);
+	            list = NULL ;
+			}
 		}
 		if (get_flags (extwm_hints->flags, EXTWM_DESKTOP))
 			set_32bit_property (w, _XA_NET_WM_DESKTOP, XA_CARDINAL, extwm_hints->desktop);
 		if (get_flags (extwm_hints->flags, EXTWM_PID))
-			set_32bit_property (w, _XA_NET_WM_PID, XA_CARDINAL, extwm_hints->pid);
+			set_32bit_property (w, _XA_NET_WM_PID, XA_CARDINAL, extwm_hints->pid); 
+		if (get_flags (extwm_hints->flags, EXTWM_WINDOW_OPACITY))
+			set_32bit_property (w, _XA_NET_WM_WINDOW_OPACITY, XA_CARDINAL, extwm_hints->window_opacity); 
+		if (get_flags (extwm_hints->flags, EXTWM_ICON))
+			set_32bit_proplist (w, _XA_NET_WM_ICON, XA_CARDINAL, extwm_hints->icon, extwm_hints->icon_length);
 	}
 }
 
@@ -1365,6 +1509,15 @@ set_client_hints (Window w, XWMHints * hints, XSizeHints * size_hints, ASFlagTyp
 	}
 }
 
+void
+set_client_cmd (Window w)
+{
+	if ( w != None && MyArgsPtr->saved_argv && MyArgsPtr->saved_argc > 0 )
+	{
+		XSetCommand(dpy, w, MyArgsPtr->saved_argv, MyArgsPtr->saved_argc);	
+	}
+}
+
 /***************************************************************************
  * ICCCM Client Messages - Section 4.2.8 of the ICCCM dictates that all
  * client messages will have the following form:
