@@ -22,6 +22,7 @@ typedef enum
 	HINTS_Transient,
 	HINTS_Motif,
     HINTS_Gnome,
+	HINTS_KDE,
     HINTS_ExtendedWM,
     HINTS_XResources,
     HINTS_ASDatabase,
@@ -172,11 +173,16 @@ extern Atom _XA_NET_WM_STATE_HIDDEN;
 extern Atom _XA_NET_WM_STATE_FULLSCREEN;
 extern Atom _XA_NET_WM_STATE_ABOVE;
 extern Atom _XA_NET_WM_STATE_BELOW;
+extern Atom _XA_NET_WM_STATE_DEMANDS_ATTENTION;
 
-#define MAX_NET_WM_STATES   7
+#define MAX_NET_WM_STATES   12
 
 extern Atom _XA_NET_WM_PID;
 extern Atom _XA_NET_WM_PING;
+extern Atom _XA_NET_WM_ICON;
+
+extern Atom _XA_NET_WM_WINDOW_OPACITY;
+#define NET_WM_WINDOW_OPACITY_OPAQUE		0xffffffff
 
 extern Atom _XA_AS_MOVERESIZE;
 
@@ -307,6 +313,15 @@ This are placed By Window Manager :
     height_inc           CARD32
 */
 
+#ifndef 	UrgencyHint
+# ifndef 	XUrgencyHint
+#  define 	UrgencyHint		256
+# else
+#  define 	UrgencyHint		XUrgencyHint
+# endif
+#endif
+
+
 /************************************************************************/
 /* 		Motif WM window hints					*/
 /************************************************************************/
@@ -452,6 +467,7 @@ GnomeHints;
 
  Protocols :
     _NET_WM_PING
+ 	_NET_WM_ICON
 
 */
 
@@ -463,38 +479,12 @@ typedef struct ExtendedWMHints
 #define EXTWM_VISIBLE_ICON_NAME (0x01<<3)
 #define EXTWM_DESKTOP           (0x01<<4)
 
-#define EXTWM_TypeDesktop       (0x01<<5)
-#define EXTWM_TypeDock          (0x01<<6)
-#define EXTWM_TypeToolbar       (0x01<<7)
-#define EXTWM_TypeMenu          (0x01<<8)
-#define EXTWM_TypeDialog        (0x01<<9)
-#define EXTWM_TypeNormal        (0x01<<10)
-#define EXTWM_TypeUtility       (0x01<<11)
-#define EXTWM_TypeSplash        (0x01<<12)
-#define EXTWM_TypeASModule      (0x01<<13)
-#define EXTWM_TypeEverything    (EXTWM_TypeDesktop|EXTWM_TypeDock|EXTWM_TypeToolbar| \
-                                 EXTWM_TypeMenu|EXTWM_TypeDialog|EXTWM_TypeNormal| \
-								 EXTWM_TypeUtility|EXTWM_TypeSplash|EXTWM_TypeASModule)
-
-#define EXTWM_StateModal        (0x01<<16)
-#define EXTWM_StateSticky       (0x01<<17)
-#define EXTWM_StateMaximizedV   (0x01<<18)
-#define EXTWM_StateMaximizedH   (0x01<<19)
-#define EXTWM_StateShaded       (0x01<<21)
-#define EXTWM_StateSkipTaskbar  (0x01<<22)
-#define EXTWM_StateSkipPager    (0x01<<23)
-#define EXTWM_StateHidden    	(0x01<<24)
-#define EXTWM_StateFullscreen	(0x01<<25)
-#define EXTWM_StateAbove	 	(0x01<<26)
-#define EXTWM_StateBelow	 	(0x01<<27)
-#define EXTWM_StateEverything   (EXTWM_StateModal|EXTWM_StateSticky|EXTWM_StateMaximizedV| \
-                                 EXTWM_StateMaximizedH|EXTWM_StateShaded| \
-                                 EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager | \
-								 EXTWM_StateHidden|EXTWM_StateFullscreen| \
-								 EXTWM_StateAbove|EXTWM_StateBelow)
-
-#define EXTWM_PID               (0x01<<29)
-#define EXTWM_DoesWMPing        (0x01<<30)
+#define EXTWM_TypeSet	        (0x01<<5)
+#define EXTWM_StateSet	        (0x01<<6)
+#define EXTWM_ICON              (0x01<<7)
+#define EXTWM_PID               (0x01<<8)
+#define EXTWM_DoesWMPing        (0x01<<9)
+#define EXTWM_WINDOW_OPACITY    (0x01<<10)
 
 /* actions to be performed on window state upon client request :*/
 #define EXTWM_StateRemove   0    /* remove/unset property */
@@ -502,16 +492,63 @@ typedef struct ExtendedWMHints
 #define EXTWM_StateToggle   2    /* toggle property  */
 
   ASFlagType flags;
+  
+#define EXTWM_TypeDesktop       (0x01<<0)
+#define EXTWM_TypeDock          (0x01<<1)
+#define EXTWM_TypeToolbar       (0x01<<2)
+#define EXTWM_TypeMenu          (0x01<<3)
+#define EXTWM_TypeDialog        (0x01<<4)
+#define EXTWM_TypeNormal        (0x01<<5)
+#define EXTWM_TypeUtility       (0x01<<6)
+#define EXTWM_TypeSplash        (0x01<<7)
+#define EXTWM_TypeASModule      (0x01<<8)
+#define EXTWM_TypeEverything    (EXTWM_TypeDesktop|EXTWM_TypeDock|EXTWM_TypeToolbar| \
+                                 EXTWM_TypeMenu|EXTWM_TypeDialog|EXTWM_TypeNormal| \
+								 EXTWM_TypeUtility|EXTWM_TypeSplash|EXTWM_TypeASModule)
 
+  ASFlagType type_flags;
+
+#define EXTWM_StateModal        (0x01<<16)
+#define EXTWM_StateSticky       (0x01<<17)
+#define EXTWM_StateMaximizedV   (0x01<<18)
+#define EXTWM_StateMaximizedH   (0x01<<19)
+#define EXTWM_StateShaded       (0x01<<20)
+#define EXTWM_StateSkipTaskbar  (0x01<<21)
+#define EXTWM_StateSkipPager    (0x01<<22)
+#define EXTWM_StateHidden    	(0x01<<23)
+#define EXTWM_StateFullscreen	(0x01<<24)
+#define EXTWM_StateAbove	 	(0x01<<25)
+#define EXTWM_StateBelow	 	(0x01<<26)
+#define EXTWM_StateDemandsAttention	(0x01<<27)
+#define EXTWM_StateEverything   (EXTWM_StateModal|EXTWM_StateSticky|EXTWM_StateMaximizedV| \
+                                 EXTWM_StateMaximizedH|EXTWM_StateShaded| \
+                                 EXTWM_StateSkipTaskbar|EXTWM_StateSkipPager | \
+								 EXTWM_StateHidden|EXTWM_StateFullscreen| \
+								 EXTWM_StateAbove|EXTWM_StateBelow|EXTWM_StateDemandsAttention)
+
+  ASFlagType state_flags;
+  
   XTextProperty *name;
   XTextProperty *icon_name;
   XTextProperty *visible_name;
   XTextProperty *visible_icon_name;
   CARD32 desktop;
   CARD32 pid;
+  CARD32 *icon;
+  long icon_length;
+  CARD32 window_opacity ;
 }
 ExtendedWMHints;
 
+typedef struct KDEHints
+{
+#define KDE_DesktopWindow              (0x01<<0)
+#define KDE_SysTrayWindowFor           (0x01<<1)
+	
+	ASFlagType flags ;
+	Window systray_window_for;
+}KDEHints;
+
 /***********************************************************************/
 /*    AfterStep structure to collect all the hints set on window :     */
 /***********************************************************************/
@@ -572,6 +609,7 @@ typedef struct ASRawHints
 
   /* WM-specs (new Gnome) Hints : */
   ExtendedWMHints extwm_hints;
+  KDEHints	kde_hints;
 
   ASFlagType  hints_types ;
 
@@ -620,6 +658,7 @@ Bool get_extwm_state_flags (Window w, ASFlagType *flags);
 /****************** Setting properties - updating hints : ****************/
 /*************************************************************************/
 void set_client_state( Window w, struct ASStatusHints *status );
+void set_extwm_urgency_state (Window w, Bool set );
 void set_client_desktop( Window w, int ext_desk );
 void set_client_names( Window w, char *name, char *icon_name, char *res_class, char *res_name );
 void set_client_protocols (Window w, ASFlagType protocols, ASFlagType extwm_protocols);
@@ -628,6 +667,7 @@ void set_gnome_hints( Window w, GnomeHints *gnome_hints );
 
 void set_client_hints( Window w, XWMHints *hints, XSizeHints *size_hints, ASFlagType protocols,
 				  	   ExtendedWMHints *extwm_hints );
+void set_client_cmd (Window w);
 
 
 void send_wm_protocol_request (Window w, Atom request, Time timestamp);
