@@ -53,6 +53,21 @@
 /**********************************************************************/
 /*  Gadget local variables :                                         */
 /**********************************************************************/
+struct ASWinTab;
+
+typedef struct ASWinTabGroup
+{
+	Bool 	pattern_is_tail;
+	int 	pattern_length;
+	char*	pattern;
+	INT32 	pattern_encoding ;
+	Bool 	selected;		
+	
+	Window  selected_client;
+	int		seqno;
+	
+}ASWinTabGroup;
+
 typedef struct ASWinTab
 {
 	char 			*name ;
@@ -72,6 +87,11 @@ typedef struct ASWinTab
 	
 	time_t      last_selected ;
 
+	ASWinTabGroup  *group;
+	Bool 			group_owner;
+	int 			group_seqno;
+
+	int calculated_width;
 }ASWinTab;
 
 typedef struct {
@@ -105,8 +125,7 @@ typedef struct {
 	int rows ;
 	int row_height ;
 
-    int selected_tab ;
-    int pressed_tab ;
+    Window selected_client;
 
 	int win_width, win_height ;
 
@@ -155,6 +174,50 @@ char *exclude_pattern_override = NULL ;
 char *title_override = NULL, *icon_title_override = NULL ;
 char *border_color_override = NULL ;
 
+static inline int 
+find_tab_for_client (Window client)
+{
+    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+    int i = PVECTOR_USED(WinTabsState.tabs) ;
+
+	if (client)
+	    while( --i >= 0 ) if (tabs[i].client == client) return i;
+	return -1;
+}
+
+static inline int 
+find_tab_pressed ()
+{
+    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+    int i = PVECTOR_USED(WinTabsState.tabs) ;
+
+    while( --i >= 0 ) if (IsASTBarPressed(tabs[i].bar)) return i;
+	return -1;
+}
+
+static inline int 
+find_group_owner (ASWinTabGroup *group)
+{
+    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+    int i = PVECTOR_USED(WinTabsState.tabs) ;
+
+	if (group)
+	    while( --i >= 0 ) if (tabs[i].group == group && tabs[i].group_owner) return i;
+	return -1;
+}
+
+static inline int 
+find_tab_for_group (ASWinTabGroup *group, int after_index)
+{
+    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+	int i = after_index;
+    int max_i = PVECTOR_USED(WinTabsState.tabs) ;
+
+	if (group)
+	    while (++i < max_i) if (tabs[i].group == group && !tabs[i].group_owner) return i;
+	return -1;
+}
+
 void CheckConfigSanity(const char *pattern_override, const char *exclude_pattern_override, 
 					   const char *title_override, const char *icon_title_override);
 void SetWinTabsLook();
@@ -193,6 +256,13 @@ Bool check_unswallowed_client( Window client );
 void delete_tab( int index );
 void set_frame_background( ASWinTab *aswt );
 
+ASWinTabGroup *check_belong_to_group (ASWinTabGroup *group, const char *name, INT32 name_encoding);
+int remove_from_group (ASWinTabGroup *group, int t);
+void add_to_group (ASWinTabGroup *group, int t);
+void check_create_new_group();
+void check_tab_grouping (int t);
+void fix_grouping_order();
+
 /* above function may also return : */
 #define BANNER_TAB_INDEX -1		   
 #define INVALID_TAB_INDEX -2
@@ -310,7 +380,7 @@ main( int argc, char **argv )
 
 	SendInfo ("Send_WindowList", 0);
 
-    WinTabsState.selected_tab = WinTabsState.pressed_tab = -1 ;
+    WinTabsState.selected_client = None ;
 	WinTabsState.tabs = create_asvector( sizeof(ASWinTab) );
 
     WinTabsState.main_window = make_wintabs_window();
@@ -627,6 +697,9 @@ GetOptions (const char *filename)
     if( get_flags(config->set_flags, WINTABS_V_SPACING) )
         Config->v_spacing = config->v_spacing;
 
+    if( config->GroupNameSeparator )
+        set_string( &(Config->GroupNameSeparator), mystrdup(config->GroupNameSeparator) );
+
     if( Config->balloon_conf )
         Destroy_balloonConfig( Config->balloon_conf );
     Config->balloon_conf = config->balloon_conf ;
@@ -696,13 +769,20 @@ on_tabs_canvas_config()
 	int tabs_num  = PVECTOR_USED(WinTabsState.tabs);
     ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
 	int tabs_changes = handle_canvas_config( WinTabsState.tabs_canvas );
+	ASCanvas *tc = WinTabsState.tabs_canvas;
 
     if( tabs_changes != 0 )
 	{
         int i = tabs_num ;
 		Bool rerender_tabs = False ;
+		
+		/* no need to redraw if we were moved outside of the visible viewport */
+		if (tc->root_x > Scr.MyDisplayWidth || tc->root_y > Scr.MyDisplayHeight
+			|| tc->root_x + (int)tc->width <= 0 ||	tc->root_y + (int)tc->height <= 0)
+			return 0;
 
-		safe_asimage_destroy( Scr.RootImage );	  
+		safe_asimage_destroy (Scr.RootImage);	  
+		Scr.RootImage = NULL;
         set_root_clip_area(WinTabsState.tabs_canvas );
 
 		rerender_tabs = update_astbar_transparency(WinTabsState.banner.bar, WinTabsState.tabs_canvas, True);
@@ -790,12 +870,13 @@ DispatchEvent (ASEvent * event)
 						rerender_tabs = on_tabs_canvas_config();
                             
                         while( --i >= 0 ) 
-                        {
-                            handle_canvas_config( tabs[i].client_canvas );
-                            send_swallowed_configure_notify(&(tabs[i]));
-							if (get_flags (WinTabsState.flags, ASWT_Transparent))
-								XClearArea (dpy, tabs[i].frame_canvas->w, 0, 0, 0, 0, True);
-                        }    
+							if (!tabs[i].group_owner)
+							{
+	                            handle_canvas_config( tabs[i].client_canvas );
+    	                        send_swallowed_configure_notify(&(tabs[i]));
+								if (get_flags (WinTabsState.flags, ASWT_Transparent) && tabs[i].frame_canvas)
+									XClearArea (dpy, tabs[i].frame_canvas->w, 0, 0, 0, 0, True);
+							}
                     }    
                 }else if( event->w == WinTabsState.tabs_window ) 
                 {
@@ -808,20 +889,19 @@ DispatchEvent (ASEvent * event)
         case KeyPress:
 			{
 				XKeyEvent *xk = &(event->x.xkey);	
+				int selected_tab = find_tab_for_client (WinTabsState.selected_client);
 				if( xk->keycode == WINTABS_SWITCH_KEYCODE && xk->state == WINTABS_SWITCH_MOD ) 
 				{	
-					if( WinTabsState.selected_tab+1 < PVECTOR_USED( WinTabsState.tabs ) )
-				 		select_tab( WinTabsState.selected_tab+1 );    		
+					if( selected_tab+1 < PVECTOR_USED( WinTabsState.tabs ) )
+				 		select_tab( selected_tab+1 );    		
 					else
 						select_tab( 0 );    		   
 					/* XBell (dpy, event->scr->screen); */
 				}else if( xk->window != WinTabsState.tabs_window ) 
 				{	
-					if( WinTabsState.selected_tab >= 0 && WinTabsState.selected_tab < PVECTOR_USED( WinTabsState.tabs ) )
+					if (selected_tab >= 0)
     				{
-        				ASWinTab *old_tab =  PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+WinTabsState.selected_tab ;
-	
-						xk->window = old_tab->client;
+						xk->window = WinTabsState.selected_client;
         	    		XSendEvent (dpy, xk->window, False, KeyPressMask, &(event->x));
 					}
 				}
@@ -881,7 +961,7 @@ DispatchEvent (ASEvent * event)
                 int i = PVECTOR_USED(WinTabsState.tabs);
                 ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
 				while( --i >= 0 ) 
-					if( event->w == tabs[i].frame_canvas->w ) 
+					if( tabs[i].frame_canvas && event->w == tabs[i].frame_canvas->w ) 
 					{
 						XExposeEvent *xexp = &(event->x.xexpose);
 						XFillRectangle(dpy, tabs[i].frame_canvas->w, Scr.DrawGC, xexp->x, xexp->y, xexp->width, xexp->height);
@@ -1093,7 +1173,7 @@ show_banner_buttons()
 void
 set_frame_background( ASWinTab *aswt )
 {
-	Window w = aswt?aswt->frame_canvas->w:WinTabsState.main_window;
+	Window w = (aswt && aswt->frame_canvas)?aswt->frame_canvas->w:WinTabsState.main_window;
 	if( get_flags( WinTabsState.flags, ASWT_Transparent ) )
 	{
 		XSetWindowBackgroundPixmap( dpy, w, ParentRelative);
@@ -1238,6 +1318,7 @@ set_tab_title( ASWinTab *aswt )
 	int align = Config->name_aligment ;
 	int h_spacing = Config->h_spacing ;
 	int v_spacing = Config->v_spacing ;
+	char *display_name = aswt->name;
 
 	if( WinTabsState.tbar_props )
 	{
@@ -1250,7 +1331,27 @@ set_tab_title( ASWinTab *aswt )
 	}	 
 	
 	delete_astbar_tile( aswt->bar, 0 );
-	add_astbar_label( aswt->bar, 0, 0, 0, align, h_spacing, v_spacing, aswt->name, aswt->name_encoding);
+	if (aswt->group && !aswt->group_owner)
+	{
+		int name_len = strlen (aswt->name);
+		display_name = safemalloc (name_len + 16);
+		name_len -= aswt->group->pattern_length;
+		if (name_len == 0)
+		{
+			sprintf (display_name, "#%d", aswt->group_seqno);
+		}else
+		{
+			if (aswt->group->pattern_is_tail)
+				strncpy (display_name, aswt->name, name_len);
+			else
+				strcpy (display_name, aswt->name + aswt->group->pattern_length);
+			display_name[name_len] = '\0';
+		}
+	}
+
+	add_astbar_label( aswt->bar, 0, 0, 0, align, h_spacing, v_spacing, display_name, aswt->name_encoding);
+	if (display_name != aswt->name)
+		free (display_name);
 }
 
 ASWinTab *
@@ -1258,8 +1359,7 @@ add_tab( Window client, const char *name, INT32 encoding )
 {
 	ASWinTab aswt ;
 
-	if( AS_ASSERT(client) )
-		return NULL ;
+	memset (&aswt, 0x00, sizeof(aswt));
 	aswt.client = client ;
 	aswt.name = mystrdup(name);
 	aswt.name_encoding = encoding;
@@ -1279,13 +1379,17 @@ delete_tab( int index )
 {
 	int tabs_num = PVECTOR_USED(WinTabsState.tabs) ;
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
-	int tab_to_select = -1 ; 
+	Window client_to_select = None ; 
+	int owner_index = -1;
+
 	if( index >= tabs_num ) 
 		return ;
-    if( WinTabsState.selected_tab == index ) 
+
+    if ( WinTabsState.selected_client == tabs[index].client) 
     {    
-		int i;
-        WinTabsState.selected_tab = -1 ;
+		int i, tab_to_select = -1;
+		
+        WinTabsState.selected_client = None ;
 		for( i = 0 ; i < tabs_num ; ++i ) 
 			if( i != index )
 			{
@@ -1294,52 +1398,89 @@ delete_tab( int index )
 						continue;
 				tab_to_select = i ;
 			}
-    }
-    if( WinTabsState.pressed_tab == index ) 
-    {    
-        WinTabsState.pressed_tab = -1 ;
+		client_to_select = tabs[tab_to_select].client;
     }
     destroy_astbar( &(tabs[index].bar) );
 	/* we still want to receive DestroyNotify events */
-    XSelectInput (dpy, tabs[index].client, StructureNotifyMask);
-    destroy_ascanvas( &(tabs[index].client_canvas) );
-	XDestroyWindow(dpy, tabs[index].frame_canvas->w );
-	destroy_ascanvas( &(tabs[index].frame_canvas) );
-    XRemoveFromSaveSet (dpy, tabs[index].client);
-    if( tabs[index].name ) 
-        free( tabs[index].name );
-    vector_remove_index( WinTabsState.tabs, index );
+	if (tabs[index].client)
+	{
+    	XSelectInput (dpy, tabs[index].client, StructureNotifyMask);
+	    XRemoveFromSaveSet (dpy, tabs[index].client);
+	}
+	if (tabs[index].client_canvas)
+	    destroy_ascanvas( &(tabs[index].client_canvas) );
 
+	if (tabs[index].frame_canvas)
+	{
+		XDestroyWindow(dpy, tabs[index].frame_canvas->w );
+		destroy_ascanvas( &(tabs[index].frame_canvas) );
+	}
+    if (tabs[index].name) 
+        free (tabs[index].name);
+
+	if (tabs[index].group && !tabs[index].group_owner) 
+	{
+		int i;
+		int members_left = 0;
+		for (i = 0 ; i < tabs_num ; ++i) 
+			if ( i != index && tabs[i].group == tabs[index].group)
+			{
+				if (tabs[i].group_owner)
+					owner_index = i;
+				else
+					++members_left;
+			}
+		if (members_left)
+		{
+			if (client_to_select)
+		        set_astbar_focused(tabs[owner_index].bar, WinTabsState.tabs_canvas, False);
+			owner_index = -1;
+		}
+		
+	}
+
+    vector_remove_index (WinTabsState.tabs, index);
+	if (owner_index > index)
+		--owner_index;
+
+	if (owner_index >= 0)
+	    vector_remove_index (WinTabsState.tabs, owner_index);
+	
 	if( PVECTOR_USED(WinTabsState.tabs) == 0 )
 	{
-		if( get_module_out_fd() < 0 ) 
-			DeadPipe(0);
-		show_hint( False );
-	}else if( tab_to_select >= 0 )
-		select_tab( tab_to_select );
+		if (get_module_out_fd () < 0) 
+			DeadPipe (0);
+		show_hint (False);
+	}else if (client_to_select != None)
+		select_tab (find_tab_for_client (client_to_select));
 }    
 
+static inline int 
+get_restricted_width (ASWinTab *tab, int max_width)
+{
+    if( tab->calculated_width < Config->min_tab_width ) 
+        return Config->min_tab_width ;
+    return ( tab->calculated_width > max_width )?max_width:tab->calculated_width;
+}
+
 void
 place_tabs_line( ASWinTab *tabs, int x, int y, int first, int last, int spare, int max_width, int tab_height )
 {
     int i ;
-    int delta = 0;
-	
-	if( last+1 > first  ) 
-		delta = spare / (last+1-first) ;
 
     for( i = first ; i <= last ; ++i ) 
     {
-        int width  = calculate_astbar_width( tabs[i].bar );
-        if( width < Config->min_tab_width ) 
-            width = Config->min_tab_width ;
-        if( width > max_width )
-            width = max_width ;
-        width += delta ;
-        spare -= delta ; 
-        if( i < last ) 
-            delta = spare / (last - i) ;
+		int delta = spare / (last+1-i) ;
+        int width  = get_restricted_width (tabs+i, max_width);
 
+LOCAL_DEBUG_OUT ("i = %d, name = \"%s\", x = %d, width = %d", i, tabs[i].name, x, width );
+		if (!tabs[i].group_owner || delta < 0)
+		{
+        	width += delta ;
+	        spare -= delta ; 
+		}
+
+LOCAL_DEBUG_OUT ("i = %d, name = \"%s\", x = %d, width = %d", i, tabs[i].name, x, width );
         set_astbar_size( tabs[i].bar, width, tab_height );
         move_astbar( tabs[i].bar, WinTabsState.tabs_canvas, x, y );
         x += width ;
@@ -1397,6 +1538,7 @@ rearrange_tabs( Bool dont_resize_window )
     int max_x = mc->width ; 
     int max_y = mc->height ; 
     int max_width = Config->max_tab_width ; 
+    int max_group_owner_width = 0; 
     int start = 0, start_x ;
 
 	if( tabs_num == 0 ) 
@@ -1467,21 +1609,40 @@ rearrange_tabs( Bool dont_resize_window )
     set_astbar_size( WinTabsState.banner.bar, x, tab_height );
     move_astbar( WinTabsState.banner.bar, WinTabsState.tabs_canvas, 0, 0 );
 
+    for( i = 0 ; i < tabs_num ; ++i ) 
+	{
+		int width = calculate_astbar_width( tabs[i].bar );
+
+		if (tabs[i].calculated_width < width)
+			tabs[i].calculated_width = width;
+
+		if (max_group_owner_width < tabs[i].calculated_width && tabs[i].group_owner)
+			max_group_owner_width = tabs[i].calculated_width;
+	}
+#if 0 
+	/* having all the group tabs to be the same size turns out not to be such a great idea */       
+	if (max_group_owner_width > 0)
+	    for (i = 0 ; i < tabs_num ; ++i) 
+		{
+			if (tabs[i].group_owner)
+				tabs[i].calculated_width = max_group_owner_width;
+		}
+#endif	
+
 	start = 0 ;
 	start_x = x ;
     for( i = 0 ; i < tabs_num ; ++i ) 
     {    
-        int width  = calculate_astbar_width( tabs[i].bar );
-        if( width < Config->min_tab_width ) 
-            width = Config->min_tab_width ;
-        if( width > max_width )
-            width = max_width ;
-        if( x + width > max_x )
+        int width  = get_restricted_width (tabs+i, max_width);
+
+        if (x + width > max_x || (i > start+1 && tabs[i].group != tabs[i-1].group))
         {   
 			if( i  > 0 )  
             	place_tabs_line( tabs, start_x, y, start, i - 1, max_x - x, max_width, tab_height );
+
             if( y + tab_height > max_y ) 
                 break;
+
             y += tab_height ; 
             x = 0 ;
 			start = i ;
@@ -1545,34 +1706,61 @@ render_tabs( Bool canvas_resized )
 }
 
 void 
-select_tab( int tab )
+select_tab (int tab)
 {
-	ASWinTab *new_tab ;
+	ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
 	ASStatusHints status = {0};
 	ASFlagType default_status_flags = get_flags(WinTabsState.flags,ASWT_StateShaded)?AS_Shaded:0;
+	int current_selection;
 
 	default_status_flags |= get_flags(WinTabsState.flags,ASWT_StateSticky)?AS_Sticky:0;
 
-    if( tab == WinTabsState.selected_tab ) 
-        return ; 
-    if( WinTabsState.selected_tab >= 0 && WinTabsState.selected_tab < PVECTOR_USED( WinTabsState.tabs ) )
-    {
-        ASWinTab *old_tab =  PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+WinTabsState.selected_tab ;
-		status.flags = AS_Hidden|default_status_flags ; 
-		set_client_state (old_tab->client, &status);
-        set_astbar_focused(old_tab->bar, WinTabsState.tabs_canvas, False);
-        WinTabsState.selected_tab = -1 ;
-    }    
     if( tab < 0 || tab >= PVECTOR_USED( WinTabsState.tabs ) )
 		tab = 0 ;
-		
-    new_tab = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+tab ;
+
+	if (tabs[tab].group_owner)
+	{
+		if ((tab = find_tab_for_client (tabs[tab].group->selected_client)) < 0)
+			if ((tab = find_tab_for_group (tabs[tab].group, 0))<0)
+				return;
+	}
+	
+    if( tabs[tab].client == WinTabsState.selected_client ) 
+        return ; 
+
+	current_selection = find_tab_for_client (WinTabsState.selected_client);
+	
+    if (current_selection >= 0)
+    {
+		status.flags = AS_Hidden|default_status_flags ; 
+		set_client_state (WinTabsState.selected_client, &status);
+        set_astbar_focused(tabs[current_selection].bar, WinTabsState.tabs_canvas, False);
+        WinTabsState.selected_client = None ;
+
+		if (tabs[current_selection].group && tabs[current_selection].group != tabs[tab].group)
+		{
+			int old_group_owner = find_group_owner (tabs[current_selection].group);
+			if (old_group_owner >= 0)
+		        set_astbar_focused(tabs[old_group_owner].bar, WinTabsState.tabs_canvas, False);
+		}
+    }    
+
 	status.flags = default_status_flags ; 
-	set_client_state (new_tab->client, &status);
-    set_astbar_focused(new_tab->bar, WinTabsState.tabs_canvas, True);
-    XRaiseWindow( dpy, new_tab->frame_canvas->w );
-    WinTabsState.selected_tab = tab ;
-	new_tab->last_selected = time(NULL);
+	set_client_state (tabs[tab].client, &status);
+    set_astbar_focused(tabs[tab].bar, WinTabsState.tabs_canvas, True);
+	if (tabs[tab].frame_canvas)
+	    XRaiseWindow( dpy, tabs[tab].frame_canvas->w );
+
+    WinTabsState.selected_client = tabs[tab].client;
+
+	if (tabs[tab].group)
+	{
+		int new_group_owner = find_group_owner (tabs[tab].group);
+		tabs[tab].group->selected_client = tabs[tab].client;
+        set_astbar_focused(tabs[new_group_owner].bar, WinTabsState.tabs_canvas, True);
+	}
+
+	tabs[tab].last_selected = time(NULL);
 	ASSync(False);
 	update_focus();
 }    
@@ -1580,19 +1768,18 @@ select_tab( int tab )
 void 
 press_tab( int tab )
 {
-    if( tab == WinTabsState.pressed_tab ) 
+	int curr = find_tab_pressed ();
+
+    if (tab == curr) 
         return ; 
-    if( WinTabsState.pressed_tab >= 0 && WinTabsState.pressed_tab < PVECTOR_USED( WinTabsState.tabs ) )
+    if (curr >= 0)
     {
-        ASWinTab *old_tab =  PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+WinTabsState.pressed_tab ;
-        set_astbar_pressed(old_tab->bar, WinTabsState.tabs_canvas, False);
-        WinTabsState.pressed_tab = -1 ;
+        set_astbar_pressed(PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)[curr].bar, WinTabsState.tabs_canvas, False);
     }    
     if( tab >= 0 && tab < PVECTOR_USED( WinTabsState.tabs ) )
     {
         ASWinTab *new_tab = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs)+tab ;
         set_astbar_pressed(new_tab->bar, WinTabsState.tabs_canvas, True);
-        WinTabsState.pressed_tab = tab ;
     }
 }    
 
@@ -1602,11 +1789,13 @@ press_tab( int tab )
 void
 check_wm_protocols( ASWinTab *aswt )
 {
-	ASRawHints hints ;
-	memset( &hints, 0x00, sizeof(ASRawHints));
- 	read_wm_protocols ( &hints, aswt->client);
-	aswt->wm_protocols = hints.wm_protocols ;
-	
+	if (aswt->client)
+	{
+		ASRawHints hints ;
+		memset( &hints, 0x00, sizeof(ASRawHints));
+	 	read_wm_protocols ( &hints, aswt->client);
+		aswt->wm_protocols = hints.wm_protocols ;
+	}
 }
 
 void
@@ -1620,7 +1809,7 @@ do_swallow_window( ASWindowData *wd )
 	ASWinTab *aswt = NULL ;
 	int gravity = NorthWestGravity ;
 
-	if( wd->client == WinTabsState.main_window )
+	if( wd->client == WinTabsState.main_window || wd->client == None)
 		return;
 
 	/* we have a match */
@@ -1733,10 +1922,16 @@ do_swallow_window( ASWindowData *wd )
 	map_canvas_window( nc, True );
     send_swallowed_configure_notify(aswt);
     
-    select_tab( PVECTOR_USED(WinTabsState.tabs)-1 );
-
 	if( !handle_tab_name_change( wd->client ) )
+	{
+		check_tab_grouping (PVECTOR_USED(WinTabsState.tabs)-1);
 	    rearrange_tabs( False );
+	}
+
+	/* have to do that AFTER we done with grouping, 
+	 * so that group owner would get selected : 
+	 */
+    select_tab (find_tab_for_client (wd->client));
 
     ASSync(False);
     ungrab_server();
@@ -1835,30 +2030,22 @@ void unregister_client( Window client );
 void 
 on_destroy_notify(Window w)
 {
-    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
-    int i = PVECTOR_USED(WinTabsState.tabs) ;
+    int i = find_tab_for_client (w);
 
 	unregister_client(w);
-    while( --i >= 0 ) 
-        if( tabs[i].client == w ) 
-        {
-            delete_tab( i );
-            rearrange_tabs( False );
-            return ; 
-        }    
+	if (i >= 0)
+    {
+    	delete_tab( i );
+        rearrange_tabs( False );
+    }    
 }    
 
 void 
 on_unmap_notify(Window w)
 {
-    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
-    int i = PVECTOR_USED(WinTabsState.tabs) ;
-    while( --i >= 0 ) 
-        if( tabs[i].client == w ) 
-        {
-			unswallow_tab( i );
-            return ; 
-        }    
+	int i = find_tab_for_client (w);
+	if (i >= 0)
+		unswallow_tab( i );
 }    
 
 
@@ -1902,11 +2089,11 @@ void send_swallow_command()
 
 void close_current_tab()
 {
-	int curr = WinTabsState.selected_tab ;
+	int curr = find_tab_for_client(WinTabsState.selected_client);
     int tabs_num  =  PVECTOR_USED(WinTabsState.tabs);
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
 	
-	if( tabs_num > 0 && curr >= 0 && curr < tabs_num ) 
+	if( tabs_num > 0 && curr >= 0 && tabs[curr].client) 
 	{
 		if( tabs[curr].closed ) 
 			XKillClient( dpy, tabs[curr].client); 	
@@ -1923,7 +2110,7 @@ Bool unswallow_tab(int t)
     int tabs_num  =  PVECTOR_USED(WinTabsState.tabs);
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
 	LOCAL_DEBUG_OUT( "tab = %d, tabs_num = %d", t, tabs_num );
-	if( tabs_num > 0 && t >= 0 && t < tabs_num ) 
+	if( t >= 0 && t < tabs_num && tabs[t].client) 
 	{
 		/* XGrabKey( dpy, WINTABS_SWITCH_KEYCODE, Mod1Mask|Mod2Mask, w, True, GrabModeAsync, GrabModeAsync); */
 		XDeleteProperty( dpy, tabs[t].client, _XA_NET_WM_STATE );
@@ -1941,18 +2128,17 @@ Bool unswallow_tab(int t)
 
 Bool unswallow_current_tab()
 {
-	return unswallow_tab(WinTabsState.selected_tab);
+	return unswallow_tab(find_tab_for_client(WinTabsState.selected_client));
 }	 
 
 void 
 update_focus()
 {
-	int curr = WinTabsState.selected_tab ;
-    int tabs_num  =  PVECTOR_USED(WinTabsState.tabs);
+	int curr = find_tab_for_client(WinTabsState.selected_client);
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
 
-	LOCAL_DEBUG_OUT( "curr = %d, tabs_num = %d, focused = %ld", curr, tabs_num, get_flags(WinTabsState.flags, ASWT_StateFocused ) );	   
-	if( tabs_num > 0 && curr >= 0 && curr < tabs_num && get_flags(WinTabsState.flags, ASWT_StateFocused )) 
+	LOCAL_DEBUG_OUT( "curr = %d, tabs_num = %d, focused = %ld", curr, PVECTOR_USED(WinTabsState.tabs), get_flags(WinTabsState.flags, ASWT_StateFocused ) );	   
+	if( curr >= 0 && tabs[curr].client && get_flags(WinTabsState.flags, ASWT_StateFocused )) 
 	{
 		if( get_flags( tabs[curr].wm_protocols, AS_DoesWmTakeFocus ) )
 			send_wm_protocol_request ( tabs[curr].client, _XA_WM_TAKE_FOCUS, Scr.last_Timestamp);
@@ -1961,20 +2147,49 @@ update_focus()
 	}	
 }	 
 
+void check_tab_grouping (int t)
+{
+    ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+
+	if (!get_flags (Config->flags, WINTABS_GroupTabs))
+		return;
+		
+LOCAL_DEBUG_OUT ("group_owner = %d", tabs[t].group_owner);
+	if (!tabs[t].group_owner)
+	{
+		ASWinTabGroup *group = tabs[t].group;
+LOCAL_DEBUG_OUT ("group = %p", group);
+		if (group)
+		{
+			if (!check_belong_to_group (group, tabs[t].name, tabs[t].name_encoding))
+			{
+				t = remove_from_group (group, t);		
+				group = NULL;
+				fix_grouping_order();
+			}
+		} 
+
+LOCAL_DEBUG_OUT ("group = %p", group);
+		if (group == NULL)
+		{
+			if ((group = check_belong_to_group (NULL, tabs[t].name, tabs[t].name_encoding)) != NULL)
+			{
+				add_to_group (group, t);
+				fix_grouping_order();
+			}else 
+				check_create_new_group();
+		}
+	}
+}
+
 Bool handle_tab_name_change( Window client)
 {
     int tabs_num  =  PVECTOR_USED(WinTabsState.tabs);
     ASWinTab *tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
-	int i ;
-	Bool changed = False ;
+	int i = find_tab_for_client (client);
+	Bool changed = True ;
 
-	for( i = 0 ; i < tabs_num ; ++i ) 
-	{
-		if( tabs[i].client == client  )
-			break;	  
-	}	 
-	
-	if( i < tabs_num ) 
+	if( i >= 0 && i < tabs_num ) 
 	{
 		ASRawHints    raw;
 		ASHints       clean;
@@ -1998,9 +2213,7 @@ Bool handle_tab_name_change( Window client)
 						free( tabs[i].name );
 						tabs[i].name = mystrdup( clean.names[0] );
 						tabs[i].name_encoding = clean.names_encoding[0] ;
-						set_tab_title( &(tabs[i]) );
-						rearrange_tabs( False );
-						changed = True ;
+						changed = tabs[i].client ;
 					}	 
 				}	 
 				destroy_hints( &clean, True );
@@ -2009,6 +2222,16 @@ Bool handle_tab_name_change( Window client)
 		}
 		destroy_hints_list( &list );		
 
+LOCAL_DEBUG_OUT ("changed= %d", changed);
+		if (changed)
+		{
+			check_tab_grouping (i);
+    		tabs = PVECTOR_HEAD(ASWinTab,WinTabsState.tabs);
+			i = find_tab_for_client (client);
+			if( i >= 0  )
+				set_tab_title( &(tabs[i]) );
+			rearrange_tabs( False );
+		}
 	}	 
 	return changed ;
 }	 
@@ -2019,7 +2242,8 @@ update_tabs_desktop()
 	int i = PVECTOR_USED(WinTabsState.tabs);
     ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
 	while( --i >= 0) 
-		set_client_desktop (tabs[i].client, WinTabsState.my_desktop );	
+		if (tabs[i].client)
+			set_client_desktop (tabs[i].client, WinTabsState.my_desktop );	
 }
 
 void 
@@ -2033,22 +2257,26 @@ update_tabs_state()
 	status.flags |= get_flags(WinTabsState.flags,ASWT_StateSticky)?AS_Sticky:0;
 
 	while( --i >= 0) 
-	{	
-		if( i == WinTabsState.selected_tab ) 
-			clear_flags( status.flags, AS_Hidden );
-		else
-			set_flags( status.flags, AS_Hidden );
-		set_client_state (tabs[i].client, &status);
-	}
+		if (tabs[i].client)
+		{	
+			if( tabs[i].client == WinTabsState.selected_client ) 
+				clear_flags( status.flags, AS_Hidden );
+			else
+				set_flags( status.flags, AS_Hidden );
+			set_client_state (tabs[i].client, &status);
+		}
 }
 
 void 
 register_unswallowed_client( Window client ) 
 {
-	if( WinTabsState.unswallowed_apps == NULL ) 
-		WinTabsState.unswallowed_apps = create_ashash( 0, NULL, NULL, NULL ); 	 
+	if (client)
+	{
+		if( WinTabsState.unswallowed_apps == NULL ) 
+			WinTabsState.unswallowed_apps = create_ashash( 0, NULL, NULL, NULL ); 	 
 
-	add_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), NULL );
+		add_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), NULL );
+	}
 }
 
 void 
@@ -2063,8 +2291,281 @@ Bool
 check_unswallowed_client( Window client ) 
 {
 	ASHashData hdata = {0} ;
-	if( WinTabsState.unswallowed_apps != NULL ) 
+
+	if( client && WinTabsState.unswallowed_apps != NULL ) 
 		return (get_hash_item( WinTabsState.unswallowed_apps, AS_HASHABLE(client), &hdata.vptr ) == ASH_Success );
 	return False;
 }
 
+/****************************************************************************/
+/* WinTab grouping API :                                                    */
+int
+fix_grouping_order_for_group (int owner_index)
+{
+   	ASWinTab *tabs = PVECTOR_HEAD (ASWinTab, WinTabsState.tabs);
+	int tabs_num = PVECTOR_USED (WinTabsState.tabs);
+	int last = owner_index, next;
+	
+	while (++last < tabs_num && tabs[last].group == tabs[owner_index].group);
+
+	next = last;	
+	do{ 
+		if (++next >= tabs_num)
+			return last-1;
+	}while (tabs[next].group != tabs[owner_index].group);
+	
+	vector_relocate_elem (WinTabsState.tabs, next, last);
+	return -1;
+}
+
+
+void
+fix_grouping_order()
+{
+	int i;
+	int tabs_num;
+	do
+	{
+    	ASWinTab *tabs = PVECTOR_HEAD (ASWinTab, WinTabsState.tabs);
+		tabs_num = PVECTOR_USED (WinTabsState.tabs);
+
+		for (i = 0 ; i < tabs_num; ++i)
+			if (tabs[i].group_owner)
+				if ( (i = fix_grouping_order_for_group (i)) < 0)
+					break;
+	} while (i < tabs_num);
+}
+
+ASWinTabGroup *
+check_belong_to_group (ASWinTabGroup *group, const char *name, INT32 name_encoding)
+{
+	int tab_num = PVECTOR_USED (WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD (ASWinTab, WinTabsState.tabs);
+	int i;
+LOCAL_DEBUG_CALLER_OUT ("group = %p, name = \"%s\"", group, name);
+	if (group == NULL)
+	{
+		for (i = 0; i < tab_num; ++i)
+			if (tabs[i].group && tabs[i].group_owner)
+				if (check_belong_to_group (tabs[i].group, name, name_encoding))
+					return tabs[i].group;
+	}else
+	{
+		int len = strlen (name);
+		if (len >= group->pattern_length)
+		{
+			if (group->pattern_is_tail)
+			{
+				if (strcmp (name + len - group->pattern_length, group->pattern) == 0)
+					return group;
+			}else if (strncmp (name, group->pattern, group->pattern_length) == 0)
+				return group;
+		}
+	}
+	return NULL;
+}
+
+int
+remove_from_group (ASWinTabGroup *group, int t)
+{
+	int tab_num = PVECTOR_USED(WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
+	Window client = tabs[t].client;
+
+LOCAL_DEBUG_CALLER_OUT ("group = %p, tab = %d", group, t);
+
+	if (group && t >= 0 && t < tab_num)
+		if (tabs[t].group == group && !tabs[t].group_owner)
+		{
+			int go_idx = find_group_owner (group);
+			tabs[t].group = NULL;
+			if (go_idx >= 0)
+			{
+				if (find_tab_for_group (group, 0) < 0)
+				{
+					delete_tab (go_idx);
+					free (group->pattern);
+					free (group);
+					--tab_num;
+					if (t > go_idx)
+						--t;
+				}else if (tabs[t].client == WinTabsState.selected_client)
+				{
+					set_astbar_focused(tabs[go_idx].bar, WinTabsState.tabs_canvas, False);
+				}
+			}
+			vector_relocate_elem (WinTabsState.tabs, t, tab_num-13);
+			t = find_tab_for_client (client);
+		}
+	return t;
+}
+
+void 
+add_to_group (ASWinTabGroup *group, int t)
+{
+	int tab_num = PVECTOR_USED(WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
+
+LOCAL_DEBUG_CALLER_OUT ("group = %p, tab = %d", group, t);
+	if (group && t >= 0 && t < tab_num)
+		if (tabs[t].group == NULL)
+		{
+			tabs[t].group = group;
+			tabs[t].calculated_width = 0;
+			tabs[t].group_seqno = ++(group->seqno);
+		}
+}
+
+
+#define MIN_MATCH_LENGTH 5
+
+int 
+find_longest_common_substring (const char *str1, int len1, const char *str2, const char *token_separator)
+{
+	int len2 = strlen (str2);
+	int min_len = (len1 < len2)? len1 : len2;
+	int start = 0, tail = 0; 
+	int ts_len = token_separator ? strlen (token_separator) : 0;
+	int i;
+
+	while (start < min_len && str1[start] == str2[start]) ++start;
+	
+	if (ts_len)
+	{
+		i = start-ts_len;
+		start = 0;
+		for (; i >= 0; --i)
+			if (str1[i] == token_separator[0] && strncmp (&(str1[i]), token_separator, ts_len) == 0)
+			{
+				start = i + ts_len;
+				break;
+			}
+	}
+
+	str1 += len1-1;
+	str2 += len2-1;
+	
+	while (str1[tail] == str2[tail] && tail > -min_len) --tail;
+
+	if (ts_len)
+	{
+		i = tail;
+		tail = 0;
+		for (; i <= -ts_len+1; ++i)
+			if (str1[i] == token_separator[0] && strncmp (&(str1[i]), token_separator, ts_len) == 0)
+			{
+				tail = i;
+				break;
+			}
+	}
+	
+	return (start+tail < 0)? tail : start;
+}
+
+void 
+check_create_new_group()
+{
+	int tab_num = PVECTOR_USED(WinTabsState.tabs);
+    ASWinTab *tabs = PVECTOR_HEAD( ASWinTab, WinTabsState.tabs );
+	int i, k;
+	int best_match = 0, bm_i = 0, bm_substr_len = 0;
+	int group_owners_count = -1;
+
+	for (i = 0 ; i < tab_num ; ++i)
+		if (tabs[i].group_owner)
+			group_owners_count = i;
+
+LOCAL_DEBUG_CALLER_OUT ("group_owners_count = %d", i);
+	
+	for (i = 0; i < tab_num ; ++i)
+	{
+		if (!tabs[i].group)
+		{
+			int len_i = strlen (tabs[i].name);
+			if (len_i >= MIN_MATCH_LENGTH)
+			{
+				for (k = i+1 ; k < tab_num ; ++k)
+					if (!tabs[k].group && tabs[k].name_encoding == tabs[i].name_encoding)
+					{
+						int substr_len = find_longest_common_substring (tabs[i].name, len_i, tabs[k].name, Config->GroupNameSeparator);
+						int nm = substr_len*substr_len;
+						if (best_match < nm)
+						{
+							best_match = nm;
+							bm_i = i;
+							bm_substr_len = substr_len;
+						}
+					}
+			}
+		}
+	}
+LOCAL_DEBUG_OUT ( "bm_i = %d, bm_substr_len = %d", bm_i, bm_substr_len);
+	if (best_match >= MIN_MATCH_LENGTH*MIN_MATCH_LENGTH)
+	{
+		ASWinTabGroup *group = safecalloc (1, sizeof(ASWinTabGroup));
+		char *group_name;
+		int i;
+		ASWinTab *aswt;
+		if (bm_substr_len < 0)
+		{
+			int len = strlen (tabs[bm_i].name);
+			group->pattern_is_tail = True;
+			group->pattern_length = -bm_substr_len;
+LOCAL_DEBUG_OUT ( "len = %d", len);
+LOCAL_DEBUG_OUT ( "name = \"%s\"", tabs[bm_i].name);
+LOCAL_DEBUG_OUT ( "pattern = \"%s\"", tabs[bm_i].name + len +bm_substr_len);
+			group->pattern = mystrdup (tabs[bm_i].name + len +bm_substr_len);
+		}else
+		{
+			group->pattern_length = bm_substr_len;
+			group->pattern = mystrndup (tabs[bm_i].name, bm_substr_len);
+		}
+		group->pattern_encoding = tabs[bm_i].name_encoding;
+		group_name = safemalloc (group->pattern_length + 3);
+		
+		i = 0;
+		if (group->pattern_is_tail)
+			while (isspace (group->pattern[i]) 
+					|| group->pattern[i] == '-' 
+					|| group->pattern[i] == ':' ) ++i;
+
+		sprintf (group_name, "[%s", &(group->pattern[i]));
+
+		i = group->pattern_length - i;
+		if (!group->pattern_is_tail)
+			while ( isspace(group_name[i]) 
+					|| group_name[i] == '-' 
+					|| group_name[i] == ':' 
+			       ) --i;
+
+		group_name[i+1] = ']';
+		group_name[i+2] = '\0';
+		
+		aswt = add_tab(None, group_name, group->pattern_encoding);
+		free (group_name);
+		aswt->group = group;
+		aswt->group_owner = True;
+		
+		/* somewhat crude code to have group buttons at the beginning of the list */
+		vector_relocate_elem (WinTabsState.tabs, PVECTOR_USED(WinTabsState.tabs)-1, group_owners_count+1);
+			
+		tabs = PVECTOR_HEAD(ASWinTab, WinTabsState.tabs);
+		tab_num = PVECTOR_USED(WinTabsState.tabs);
+		
+		/* now we need to add all applicable tabs to the new group */
+		for (i = 0 ; i < tab_num ; ++i)
+			if (!tabs[i].group && check_belong_to_group (group, tabs[i].name, tabs[i].name_encoding))
+			{
+				add_to_group (group, i);
+				set_tab_title( &(tabs[i]) );
+				if (tabs[i].client == WinTabsState.selected_client)
+				{
+					WinTabsState.selected_client = None;
+					select_tab (i);
+				}
+			}
+		fix_grouping_order();
+	}
+}
+
+
