@@ -41,6 +41,12 @@
 /* our status */
 ASFlagType AfterStepState = 0; /* default status */
 
+#define ASSF_BypassAutoexec	(0x01<<0)
+ASFlagType AfterStepStartupFlags = 0;
+
+/* DBUS stuff is separated into dbus.c */
+int 		  ASDBus_fd = -1;
+
 /* Config : */
 ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
@@ -49,6 +55,10 @@ int           Module_npipes = 8;
 ASBalloonState *MenuBalloons = NULL ; 
 ASBalloonState *TitlebarBalloons = NULL ; 
 
+char *original_DISPLAY_string = NULL;
+
+char *SMClientID_string = NULL;
+
 /**************************************************************************/
 void          SetupScreen();
 void          CleanupScreen();
@@ -88,6 +98,23 @@ const char *window_id2name (Display * dpy, Window w, int *how_to_free)
 }
 #endif
 
+CommandLineOpts AfterStep_cmdl_options[3] =
+{
+	{NULL, "bypass-autoexec","Bypass running autoexec on startup", NULL, handler_set_flag, &AfterStepStartupFlags, ASSF_BypassAutoexec, 0 },
+	{NULL, "sm-client-id","Session manager client's ID", NULL, handler_set_string, &SMClientID_string, 0, CMO_HasArgs },
+    {NULL, NULL, NULL, NULL, NULL, NULL, 0 }
+};
+
+
+void
+AfterStep_usage (void)
+{
+	printf (OPTION_USAGE_FORMAT " [additional options]\n", MyName );
+	print_command_line_opt("standard_options are ", as_standard_cmdl_options, 0);
+	print_command_line_opt("additional options are ", AfterStep_cmdl_options, 0);
+	exit (0);
+}
+
 
 /**************************************************************************/
 /**************************************************************************/
@@ -104,8 +131,6 @@ main (int argc, char **argv, char **envp)
 	int start_viewport_y = 0 ;
 	int start_desk = 0 ;
 
-	Bool bypass_autoexec = False ;
-
 #ifdef LOCAL_DEBUG
 #if 0
 	LOCAL_DEBUG_OUT( "calibrating sleep_a_millisec : %s","" );
@@ -124,6 +149,10 @@ main (int argc, char **argv, char **envp)
 	_as_grab_screen_func = GrabEm;
 	_as_ungrab_screen_func = UngrabEm;
 
+	original_DISPLAY_string = getenv("DISPLAY");
+	if (original_DISPLAY_string)
+		original_DISPLAY_string = mystrdup(original_DISPLAY_string);
+
 #ifdef DEBUG_TRACE_X
 	trace_window_id2name_hook = &window_id2name;
 #endif
@@ -132,15 +161,7 @@ main (int argc, char **argv, char **envp)
 #if !HAVE_DECL_ENVIRON
 	override_environ( envp );
 #endif
-    InitMyApp( CLASS_AFTERSTEP, argc, argv, NULL, NULL, 0);
-	for( i = 1 ; i< argc ; ++i)
-	{
-		if( argv[i] != NULL &&  strcmp(argv[i],"--bypass-autoexec") == 0 )
-		{
-			bypass_autoexec = True ;
-			break;
-	    }
-	}
+    InitMyApp( CLASS_AFTERSTEP, argc, argv, AfterStep_usage, NULL, 0);
 
 	LinkAfterStepConfig();
 
@@ -169,6 +190,10 @@ main (int argc, char **argv, char **envp)
 		show_error( "Hostile X server encountered - unable to proceed :-(");
 		return 1;/* failed to accure window management selection - other wm is running */
 	}
+
+
+	ASDBus_fd = asdbus_init();
+
 	XSetWindowBackground( dpy, Scr.Root, Scr.asv->black_pixel );
 	Scr.Look.desktop_animation_tint = get_random_tint_color();
 
@@ -183,6 +208,12 @@ main (int argc, char **argv, char **envp)
 		display_progress( True, "AfterStep v.%s is starting up ...", VERSION );
 	}
 
+	if (ASDBus_fd>=0)
+	{
+    	show_progress ("Successfuly accured System DBus connection.");	
+		asdbus_RegisterSMClient(SMClientID_string);
+	}
+	
 SHOW_CHECKPOINT;
 	InitSession();
 SHOW_CHECKPOINT;
@@ -209,7 +240,7 @@ SHOW_CHECKPOINT;
         {
             if( !get_flags(MyArgs.flags, ASS_SingleScreen) )
             {
-                int pid = spawn_child( MyName, (i<MAX_USER_SINGLETONS_NUM)?i:-1, i, None, C_NO_CONTEXT, True, True, NULL );
+                int pid = spawn_child( MyName, (i<MAX_USER_SINGLETONS_NUM)?i:-1, i, NULL, None, C_NO_CONTEXT, True, True, NULL );
                 if( pid >= 0 )
                     show_progress( "\t instance of afterstep spawned with pid %d.", pid );
                 else
@@ -298,7 +329,7 @@ SHOW_CHECKPOINT;
     display_progress( True, "All done." );
     remove_desktop_cover();
 
-	if( !bypass_autoexec )
+	if( !get_flags(AfterStepStartupFlags, ASSF_BypassAutoexec))
     	DoAutoexec(get_flags( AfterStepState, ASS_Restarting));
 	
 	/* once all the windows are swallowed and placed in its proper desks - we cas restore proper
@@ -660,6 +691,38 @@ Done (Bool restart, char *command )
 		already_dead = True ;
 	}
 
+    /* lets duplicate the string so we don't accidental;y delete it while closing self down */
+    if( restart )
+	{
+		int my_name_len = strlen(MyName);
+		if( command ) 
+		{
+			if( strncmp(command, MyName, my_name_len )==0 )
+				restart_self = (command[my_name_len] == ' '|| command[my_name_len] == '\0');
+			local_command = mystrdup(command);
+		}else 
+		{		  
+        	local_command = mystrdup(MyName);
+			restart_self = True ;
+	    }
+		if (!is_executable_in_path(local_command))
+		{
+			if (!restart_self || MyArgs.saved_argv[0] == NULL)
+			{
+				show_error("Cannot restart with command \"%s\" - application is not in PATH!", local_command);
+				return;
+			}
+			free(local_command);
+			if (command)
+			{
+				local_command = safemalloc(strlen(command) + 1+ strlen(MyArgs.saved_argv[0])+1);
+				sprintf( local_command, "%s %s", MyArgs.saved_argv[0], command + my_name_len);
+			}else
+				local_command = mystrdup(MyArgs.saved_argv[0]);
+		}
+	}
+
+
 #ifdef XSHMIMAGE
 	/* may not need to do that as server may still have some of the shared 
 	 * memory and work in it */
@@ -673,22 +736,6 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
 	FlushAllQueues(); 
 	sleep_a_millisec(1000);
 
-    /* lets duplicate the string so we don't accidental;y delete it while closing self down */
-    if( restart )
-	{
-		if( command ) 
-		{
-			int my_name_len = strlen(MyName);
-
-			if( strncmp(command, MyName, my_name_len )==0 )
-				restart_self = (command[my_name_len] == ' '|| command[my_name_len] == '\0');
-			local_command = mystrdup(command);
-		}else 
-		{		  
-        	local_command = mystrdup(MyName);
-			restart_self = True ;
-	    }
-	}
 	LOCAL_DEBUG_OUT( "local_command = \"%s\", restart_self = %s", local_command, restart_self?"Yes":"No"); 
     set_flags( AfterStepState, ASS_Shutdown );
     if( restart )
@@ -716,13 +763,16 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
     /* Really make sure that the connection is closed and cleared! */
     XSync (dpy, 0);
 
+	if (ASDBus_fd>=0)
+		asdbus_shutdown();
+
 #ifdef XSHMIMAGE
 	flush_shm_cache();
 #endif
 	if (restart)
 	{
 		set_flags( MyArgs.flags, ASS_Restarting );
-        spawn_child( local_command, -1, restart_screen,
+        spawn_child( local_command, -1, restart_screen, original_DISPLAY_string,
                      None, C_NO_CONTEXT, False, restart_self, NULL );
     } else
 	{
