@@ -168,7 +168,6 @@ void
 ReloadConfig(ASFlagType what)
 {
 	ASBalloonLook *balloon_look ; 
-
 	ASModuleConfig *config = parse_asmodule_config_all( AfterStepConfigClass );
 
 	/* apply it  */
@@ -184,6 +183,7 @@ ReloadConfig(ASFlagType what)
 	set_balloon_state_look( MenuBalloons,  balloon_look );
 	destroy_balloon_look( balloon_look );
 	
+	PrintMyStyleDefinitions (config->style_defs);
 	
 	destroy_ASModule_config( config );
 }
@@ -199,6 +199,9 @@ int main(int argc, char **argv )
     ASImageManager  *old_image_manager = NULL ;
     ASFontManager   *old_font_manager  = NULL ;
     InitMyApp( CLASS_AFTERSTEP, argc, argv, NULL, NULL, 0);
+
+	AfterStepConfigClass->flags |= ASMC_HandleLookMyStyles;
+	
 	LinkAfterStepConfig();
     if( ConnectX( ASDefaultScr, 0 ) < 0  )
 	{
@@ -333,6 +336,7 @@ struct config main_config[] = {
 	{"SloppyFocus", SetFlag2, (char **)SloppyFocus, (int *)0},
     {"PagingDefault", obsolete, (char **)NULL, NULL},
     {"EdgeResistance", SetInts, (char **)&TmpFeel.EdgeResistanceScroll, &TmpFeel.EdgeResistanceMove},
+    {"EdgeResistanceToDragging", SetInts, (char **)&TmpFeel.EdgeResistanceDragScroll, NULL},
 	{"BackingStore", SetFlag2, (char **)BackingStore, (int *)0},
 	{"AppsBackingStore", SetFlag2, (char **)AppsBackingStore, (int *)0},
 	{"SaveUnders", SetFlag2, (char **)SaveUnders, (int *)0},
@@ -815,6 +819,7 @@ merge_feel( ASFeel *to, ASFeel *from )
     to->OpaqueResize = from->OpaqueResize ; 
     to->EdgeResistanceScroll = from->EdgeResistanceScroll  ; 
 	to->EdgeResistanceMove = from->EdgeResistanceMove ; 
+    to->EdgeResistanceDragScroll = from->EdgeResistanceDragScroll  ; 
     to->Xzap = from->Xzap ; 
     to->Yzap = from->Yzap ; 
     to->AutoReverse = from->AutoReverse ; 
@@ -948,16 +953,21 @@ make_styles (MyLook *look)
 
     for( i = 0 ; i < BACK_STYLES ; ++i )
 		if( MSWindowName[i] )
-			look->MSWindow[i] = mystyle_list_find (look->styles_list, MSWindowName[i]);
+			look->MSWindow[i] = mystyle_find_or_get_from_file (look->styles_list, MSWindowName[i]);
     if (look->MSWindow[BACK_DEFAULT] == NULL)
-        look->MSWindow[BACK_DEFAULT] = mystyle_list_find_or_default (look->styles_list, "default");
+		look->MSWindow[BACK_DEFAULT] = mystyle_find_or_get_from_file (look->styles_list, "default");
+	
+	/* this is the last resort : */
+    if (look->MSWindow[BACK_DEFAULT] == NULL)
+	    look->MSWindow[BACK_DEFAULT] = mystyle_list_find_or_default (look->styles_list, "default");
+		
     for( i = 0 ; i < BACK_STYLES ; ++i )
         if (look->MSWindow[i] == NULL && style_names[i] )
             look->MSWindow[i] = mystyle_list_new (look->styles_list, style_names[i]);
 
     for( i = 0 ; i < MENU_BACK_STYLES ; ++i )
 		if( MSMenuName[i] )
-			look->MSMenu[i] = mystyle_list_find (look->styles_list, MSMenuName[i]);
+			look->MSMenu[i] = mystyle_find_or_get_from_file (look->styles_list, MSMenuName[i]);
 
     for( i = 0 ; i < MENU_BACK_STYLES ; ++i )
         if (look->MSMenu[i] == NULL)
@@ -969,18 +979,19 @@ make_styles (MyLook *look)
             else
                 look->MSMenu[i] = mystyle_list_new (look->styles_list, menu_style_names[i]);
         }
-    if (mystyle_list_find (look->styles_list, "ButtonPixmap") == NULL)
+    if (mystyle_find_or_get_from_file (look->styles_list, "ButtonPixmap") == NULL)
         mystyle_list_new (look->styles_list, "ButtonPixmap");
-    if (mystyle_list_find (look->styles_list, "ButtonTitleFocus") == NULL)
+    if (mystyle_find_or_get_from_file (look->styles_list, "ButtonTitleFocus") == NULL)
         mystyle_list_new (look->styles_list, "ButtonTitleFocus");
-    if (mystyle_list_find (look->styles_list, "ButtonTitleSticky") == NULL)
+    if (mystyle_find_or_get_from_file (look->styles_list, "ButtonTitleSticky") == NULL)
         mystyle_list_new (look->styles_list, "ButtonTitleSticky");
-    if (mystyle_list_find (look->styles_list, "ButtonTitleUnfocus") == NULL)
+    if (mystyle_find_or_get_from_file (look->styles_list, "ButtonTitleUnfocus") == NULL)
         mystyle_list_new (look->styles_list, "ButtonTitleUnfocus");
 }
 
-MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagType default_title_align )
+MyFrame *add_myframe_from_def(MyLook *look, MyFrameDefinition *fd, ASFlagType default_title_align )
 {
+	ASHashTable *list = look->FramesList;
     MyFrame *frame;
     int i ;
 
@@ -1030,9 +1041,17 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
         if( fd->title_styles[i] )
+		{
             set_string(&(frame->title_style_names[i]), mystrdup(fd->title_styles[i]) );
+			/* force load the MyStyle in question */
+			mystyle_find_or_get_from_file (look->styles_list, fd->title_styles[i]);
+		}
         if( fd->frame_styles[i] )
+		{
             set_string(&(frame->frame_style_names[i]), mystrdup(fd->frame_styles[i]) );
+			/* force load the MyStyle in question */
+			mystyle_find_or_get_from_file (look->styles_list, fd->title_styles[i]);
+		}
     }
     if( get_flags( fd->set_title_attr, MYFRAME_TitleFBevelSet ) )
         frame->title_fbevel = fd->title_fbevel;
@@ -1213,7 +1232,7 @@ FixLook( MyLook *look )
 	    for( sd = MyStyleList ; sd != NULL ; sd = sd->next )
         {
             LOCAL_DEBUG_OUT( "processing MyStyleDefinition %p", sd );
-            mystyle_create_from_definition( sd );
+            mystyle_create_from_definition( look->styles_list, sd );
         }
         DestroyMyStyleDefinitions (&MyStyleList);
 	}
@@ -1277,14 +1296,14 @@ FixLook( MyLook *look )
         for( fd = MyFrameList ; fd != NULL ; fd = fd->next )
         {
             LOCAL_DEBUG_OUT( "processing MyFrameDefinition %p", fd );
-            if( (frame = add_myframe_from_def( look->FramesList, fd, default_title_align|ALIGN_VCENTER )) != NULL ) 
+            if( (frame = add_myframe_from_def( look, fd, default_title_align|ALIGN_VCENTER )) != NULL ) 
             	myframe_load ( frame, Scr.image_manager );
         }
 		if( LegacyFrameDef )
 		{
             LOCAL_DEBUG_OUT( "processing legacy MyFrameDefinition %p", LegacyFrameDef );
 			LegacyFrameDef->name = mystrdup(look->DefaultFrameName);
-            if( (frame = add_myframe_from_def( look->FramesList, LegacyFrameDef, default_title_align|ALIGN_VCENTER )) != NULL )
+            if( (frame = add_myframe_from_def( look, LegacyFrameDef, default_title_align|ALIGN_VCENTER )) != NULL )
             	myframe_load ( frame, Scr.image_manager );
 		}
         DestroyMyFrameDefinitions (&MyFrameList);
@@ -1863,21 +1882,26 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		recolor_feel_cursors( &Scr.Feel, cursor_fore, cursor_back );
 		XDefineCursor (dpy, Scr.Root, Scr.Feel.cursors[ASCUR_Default]);
 
-	    display_progress( True, "Reloading menu pixmaps :");
+	    display_progress( True, get_flags( Scr.Look.flags, MenuMiniPixmaps )?"Reloading menu pixmaps :":"Unloading menu pixmaps :");
 		if( start_hash_iteration (Scr.Feel.Popups, &i) )
             do
             {
 				MenuData *md = curr_hash_data(&i);
+				if (!get_flags (Scr.Look.flags, MenuMiniPixmaps))
+					free_menu_pmaps (md);
+				else
 				{
 					char *name = md->name;
 					Bool newline = ( count%10 == 0 );
 					if( isdigit(name[0]) )
 						if( md->first != NULL && md->first->fdata->func == F_TITLE ) 
-					    	name = md->first->item;
+				    		name = md->first->item;
 					display_progress( newline, newline?"    %s":"%s", name);
 					++count;
+
+					reload_menu_pmaps( md, get_flags(what, PARSE_BASE_CONFIG) );
 				}
-				reload_menu_pmaps( md, get_flags(what, PARSE_BASE_CONFIG) );
+
             }while( next_hash_item( &i ));
 		
 	    display_progress( True, "Advertising titlebar properties ...");
@@ -2206,7 +2230,10 @@ SetFlag2 (char *text, FILE * fd, char **arg, int *var)
 void
 SetInts (char *text, FILE * fd, char **arg1, int *arg2)
 {
-    sscanf (text, "%d%*c%d", (int *)arg1, (int *)arg2);
+	if (arg2 == NULL)
+	    sscanf (text, "%d", (int *)arg1);
+	else
+	    sscanf (text, "%d%*c%d", (int *)arg1, (int *)arg2);
 /*    LOCAL_DEBUG_OUT( "text=[%s], arg1=%p, Scr.Feel.Autoreverse = %p, res = %d", text, arg1, &(Scr.Feel.AutoReverse), *((int*)arg1) );*/
 }
 
