@@ -31,6 +31,14 @@ typedef struct ASMenuItem
 typedef struct ASMenu
 {
     unsigned long magic ;
+
+#define AS_MenuRendered		(0x01<<0)
+#define AS_MenuFocused		(0x01<<1)
+#define AS_MenuPinned		(0x01<<2)
+#define AS_MenuBalloonShown	(0x01<<3)
+#define AS_MenuNameIsUTF8	(0x01<<4)
+#define AS_MenuTitleIsUTF8	(0x01<<5)
+	ASFlagType state ; 
     char  *name ;                              /* the name of the popup */
     char  *title ;                             /* text of the first F_TITLE item */
     struct ASCanvas *main_canvas;
@@ -51,16 +59,16 @@ typedef struct ASMenu
     unsigned int icon_space, arrow_space ;
 	unsigned int scroll_bar_size ;
 
-    Bool rendered ;
-    Bool focused ;
-    Bool pinned ;
-
     struct ASWindow *owner;
 
 	struct ASTBarData *scroll_up_bar, *scroll_down_bar;
 
     Window client_window;
 
+	ASBalloon *comment_balloon ;
+	ASBalloon *item_balloon ;
+	
+	MenuData  *volitile_menu_data ; /* will be destroyed upon menu being closed */
 }ASMenu;
 
 
@@ -108,7 +116,7 @@ MenuData *update_windowList (void);
 /* menu execution code :                                                 */
 /*************************************************************************/
 void run_menu( const char *name, Window client_window  );
-void run_menu_data( MenuData *md );
+ASMenu *run_menu_data( MenuData *md );
 ASMenu *run_submenu( ASMenu *supermenu, MenuData *md, int x, int y );
 ASMenu *find_asmenu( const char *name );
 void pin_asmenu( ASMenu *menu );
