@@ -59,7 +59,12 @@ HandlePaging (int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 #ifndef NO_VIRTUAL
 	if (DoHandlePageing)
 	{
-        if ((Scr.Feel.EdgeResistanceScroll >= 10000) || ((HorWarpSize == 0) && (VertWarpSize == 0)))
+		int scroll = Scr.Feel.EdgeResistanceScroll;
+
+		if (Scr.moveresize_in_progress && Scr.Feel.EdgeResistanceDragScroll >= 0)
+			scroll = Scr.Feel.EdgeResistanceDragScroll;
+		
+        if ((scroll >= 10000) || ((HorWarpSize == 0) && (VertWarpSize == 0)))
 			return;
 
 		/* need to move the viewport */
@@ -68,7 +73,7 @@ HandlePaging (int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 			return;
 
 		total = 0;
-        while (total < (int)Scr.Feel.EdgeResistanceScroll)
+        while (total < scroll)
 		{
             register int i ;
 			sleep_a_millisec(10);
@@ -166,9 +171,26 @@ HandlePaging (int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 		{
 			if (Grab)
 				grab_server();
-			XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, *xl, *yt);
+/* Pointer warping on viewport change is considered harmfull.
+Negative side effects include: 
+1) Sometime window don't get focus due to distorted sequence on EnterNotify/LeaveNotify
+2) User feels at a loss, trying to find where the god damn pointer has jumped to. 
+If they move it to the edge of the screen - they expect it to stay at the edge of the screen.
+*/
+#if 0
+fprintf (stderr, "XCROSSING: focused = %lX active = %lX\n", Scr.Windows->focused?Scr.Windows->focused->w:0, Scr.Windows->active?Scr.Windows->active->w:0); fflush(stderr);
+fprintf (stderr, "XCROSSING: curr = %+d%+d, orig = %+d%+d\n", xroot_curr, yroot_curr, xroot_orig, yroot_orig); fflush(stderr);
+fprintf (stderr, "XCROSSING: XWarpPointer to %+d%+d\n", *xl, *yt); fflush(stderr);
+			int xroot_curr, yroot_curr;
+			if (xroot_curr == xroot_orig && yroot_curr == yroot_orig)
+#endif
+/* only want to warp pointer while move-resizing, to keep size from jumping screenwhole */
+			if (Scr.moveresize_in_progress)
+				XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, *xl, *yt);
+
 			MoveViewport (Scr.Vx + *delta_x, Scr.Vy + *delta_y, False);
             XQueryPointer (dpy, Scr.Root, &wdumm, &wdumm, xl, yt, &dumm, &dumm, &udumm);
+
 			if (Grab)
 				ungrab_server();
 		}
